# -*- coding: utf-8 -*-
"""
Provides functions for plotting
"""

from PyQt5.QtWidgets import QTableWidgetItem, QApplication
from PyQt5.QtGui import QFont, QSyntaxHighlighter, QTextCursor, QTextCharFormat, QColor
from PyQt5 import QtCore


from swat_em.config import get_phase_color, get_line_color, config
from swat_em import analyse

import numpy as np
import time
import os
import re

# WORKAROUND for pyqtgraph's printing system
#  from PyQt5.QtPrintSupport import QPrinter, QPrintDialog
#  import PyQt5.QtGui
#  PyQt5.QtGui.QPrinter = QPrinter
#  PyQt5.QtGui.QPrintDialog = QPrintDialog

import pyqtgraph as pg
import pyqtgraph.exporters

pg.setConfigOption("background", "w")
pg.setConfigOption("foreground", "k")
pg.setConfigOptions(antialias=True)


def gen_coil_lines(w, h1=0.75, h2=1.5, db1=0.1, Np1=21):
    """
    Create lines of a coil for plotting.

    Parameters
    ----------
    w :  integer
         width of the coil in slots
    h1:  float
         height of the coil side in the slot
    h2:  float
         max height of the winding overhang
    db1: float
         distance of the coil side to the middle of the slot;
         should be 0..0.2
    Np1: integer
         number of plotting points in the line of the winding overhang

    Returns
    -------
    x : list 
        x values of the coil for plotting
    y : list
        y values of the coil for plotting
    """
    x1, y1 = db1, h1
    x2, y2 = w / 2, h2
    x3, y3 = w - db1, h1
    x4, y4 = x3, -y3
    x5, y5 = x2, -x2
    x6, y6 = x1, -y1

    x = []
    y = []
    x_ = np.linspace(x1, x3, Np1)
    y_ = np.interp(x_, [x1, x2, x4], [y1, y2, y3])
    x += x_.tolist()
    y += y_.tolist()

    x += x_[::-1].tolist()
    y += (-1 * y_[::-1]).tolist()
    x.append(x[0])
    y.append(y[0])
    return x, y


def gen_slot_lines(Q, bz, hz):
    """
    Create lines of a slot for plotting.

    Parameters
    ----------
    q :  integer
         number of slots
    bz:  float
         width of the tooth -> Slot width = 1 - bz
         so bz should be around 0.5
    hz:  float
         height of the slots

    Returns
    -------
    x : list 
        x values of the slots for plotting
    y : list
        y values of the slots for plotting
    """
    x = []
    y = []
    for k in range(Q + 1):
        x1 = k - 0.5
        if k == 0:
            x += [x1, x1, x1 + bz / 2, x1 + bz / 2, x1, np.nan]
        elif k == Q:
            x += [x1 - bz / 2, x1 - bz / 2, x1, x1, x1 - bz / 2, np.nan]
        else:
            x += [
                x1 - bz / 2,
                x1 - bz / 2,
                x1 + bz / 2,
                x1 + bz / 2,
                x1 - bz / 2,
                np.nan,
            ]
        y += [-hz, hz, hz, -hz, -hz, np.nan]
    return x, y


def gen_slot_filling(Q, bz, hz):
    """
    Create lines of a slot for plotting.

    Parameters
    ----------
    q :  integer
         number of slots
    bz:  float
         width of the tooth -> Slot width = 1 - bz
         so bz should be around 0.5
    hz:  float
         height of the slots

    Returns
    -------
    x : list 
        x values of the slots for plotting
    y : list
        y values (upper lines) of the slots for plotting
    y_neg : list
        y values (lower lines) of the slots for plotting
    """
    x = []
    y = []
    for k in range(Q + 1):
        x1 = k - 0.5
        if k == 0:
            x += [x1, x1 + bz / 2, np.nan]
        elif k == Q:
            x += [x1 - bz / 2, x1, np.nan]
        else:
            x += [x1 - bz / 2, x1 + bz / 2, np.nan]
        y += [hz, hz, np.nan]
    y_neg = [-k for k in y]
    return x, y, y_neg


def group_on_nan(x, y):
    xt, yt = [[]], [[]]
    for k in range(len(x)):
        if np.isnan(x[k]) or np.isnan(y[k]):
            if len(xt[-1]) > 0:
                xt.append([])
                yt.append([])
        else:
            xt[-1].append(x[k])
            yt[-1].append(y[k])
    return xt, yt


def _pg_clear_legend(leg):
    """
    clear the legend of a pyqtgraph legend
    """
    for i in leg.items[:]:
        leg.removeItem(i[1].text)


class _slot_plot:
    sh = 0.8  # slot height
    sw = 0.75  # slot width
    so = 0.2  # slot opening

    def __init__(self, layout, widget, data):
        self.layout = layout
        self.widget = widget
        self.data = data
        self.Q = 0
        self.Qxmax = 12  # max. num of slots to plot in x-axis
        self.slot = {}
        self.devide = "h"  # slot devide h (horizontal) or v (vertical

        if self.layout is None:
            self.app = pg.mkQApp()
            self.fig = pg.PlotWidget()  # title='xyz'
        else:
            self.fig = pg.PlotWidget()
            self.layout.addWidget(self.fig)
        self.fig.setAspectLocked(lock=True, ratio=1)
        self.fig.getAxis("bottom").hide()
        self.fig.getAxis("left").hide()

    def plot_slots(self, Q):
        if Q <= 0:
            return False
        sh = self.sh  # slot height
        sw = self.sw  # slot width
        so = self.so  # slot opening
        self.dy = 1.7  # next row

        self.fig.clear()
        self.fig.disableAutoRange()  # disable because of porformance
        # (a lot of elements are plottet)

        x = [0] * 10
        y = [0] * 10
        x[0], y[0] = 0.0, 1.0
        x[1], y[1] = 0.5 - so / 2, 1.0
        x[2], y[2] = 0.5 - so / 2, sh
        x[3], y[3] = 0.5 - sw / 2, sh
        x[4], y[4] = 0.5 - sw / 2, 0.0
        x[5], y[5] = 0.5 + sw / 2, 0.0
        x[6], y[6] = 0.5 + sw / 2, sh
        x[7], y[7] = 0.5 + so / 2, sh
        x[8], y[8] = 0.5 + so / 2, 1.0
        x[9], y[9] = 1.0, 1.0

        xp = [[]]
        yp = [[]]
        i_slots = 0
        dx = 0
        dy = 0
        for k in range(Q):
            if i_slots >= self.Qxmax:
                dx = 0
                dy += -self.dy
                i_slots = 0
                xp.append([])
                yp.append([])
            for a, b in zip(x, y):
                xp[-1].append(a + dx)
                yp[-1].append(b + dy)
            dx += 1
            i_slots += 1
        for k in range(len(xp)):
            pen = pg.mkPen(color="k", width=1.5)
            curve = pg.PlotCurveItem(xp[k], yp[k], pen=pen)
            self.fig.addItem(curve)

        i_slots = 0
        dy = 0
        dx = 0
        for k in range(Q):
            if i_slots >= self.Qxmax:
                dx = 0
                dy += -self.dy
                i_slots = 0

            txt = pg.TextItem(str(k + 1), anchor=(0.5, 0.5))
            txt.setPos(0.5 + dx, -0.2 + dy)
            txt.setColor("k")
            self.fig.addItem(
                txt, ignoreBounds=True
            )  # ignore because autoRange have problems with it

            dx += 1
            i_slots += 1

        self.fig.setXRange(0, self.Qxmax)
        self.fig.autoRange()

    def plot(self, data, show=False, draw_poles=False):
        self.data = data
        self.show = show
        S = self.data.get_phases()
        Q = self.data.get_num_slots()
        P = self.data.get_num_polepairs() * 2
        self.devide = "v" if self.data.get_windingstep() == 1 else "h"

        def add_text(slot, phase, pos, wdir):
            dx = slot
            dy = self.sh / 2
            while dx > self.Qxmax:
                dx -= self.Qxmax
                dy -= self.dy
            dx = dx - 1 + 0.5

            if self.devide == "h":
                if pos == 0:  # bottom layer
                    dy -= self.sh / 4
                elif pos == 1:  # top layer
                    dy += self.sh / 4

            elif self.devide == "v":
                if pos == 0:  # right layer
                    dx += self.sw / 4
                elif pos == 1:  # left layer
                    dx -= self.sw / 4

            if wdir > 0:
                txt = "+" + str(phase)  # '$\otimes$'
            else:
                txt = "-" + str(phase)  # '$\odot$'

            text = pg.TextItem(anchor=(0.5, 0.5))
            text.setHtml("<div><b>" + txt + "</b></div>")
            #  text.setPlainText(txt)
            text.setPos(dx, dy)
            text.setColor(get_phase_color(phase - 1))
            self.fig.addItem(
                text, ignoreBounds=True
            )  # ignore because autoRange have problems with it

        for m, phase in enumerate(S):
            pos = 0
            for layer in phase:
                for cs in layer:
                    add_text(abs(cs), m + 1, pos, cs)
                pos += 1

        # draw poles
        if draw_poles:
            alpha_m = config["plt"]["magnet_alpha_m"]
            magnet_colors = config["plt"]["magnet_colors"]
            magnet_linewidth = config["plt"]["magnet_linewidth"]
            x = np.linspace(0, Q / P * alpha_m, 50)
            x += (1 - alpha_m) * Q / P / 2
            for kpole in [0, 1]:  # positive and negative Pole
                x1 = np.array([])
                for kp in range(P // 2):
                    x1 = np.append(x1, x + 2 * Q / P * kp + kpole * Q / P)
                    x1 = np.append(x1, np.nan)
                y1 = 1.1 * np.ones(len(x1))
                while np.nanmax(x1) > self.Qxmax:
                    idx = x1 > self.Qxmax
                    x1[idx] = x1[idx] - self.Qxmax
                    y1[idx] = y1[idx] - self.dy
                    idx2 = np.where(idx == True)[0][0]
                    x1[idx2] = np.nan

                pen = pg.mkPen(magnet_colors[kpole], width=magnet_linewidth)

                # Bug in pyqtgraph / PyQt5 > v5.12.1
                #  curve = pg.PlotCurveItem(x1, y1, pen=pen, connect="finite")
                #  self.fig.addItem(curve)
                for xtmp, ytmp in zip(*group_on_nan(x1, y1)):
                    curve = pg.PlotCurveItem(xtmp, ytmp, pen=pen, connect="finite")
                    self.fig.addItem(curve)

        self.fig.autoRange()
        if self.show:
            self.fig.show()
            self.app.exec_()

    def save(self, fname, res):
        if self.layout == None:
            self.app.processEvents()
        if os.path.splitext(fname)[-1].upper() == ".SVG":
            exporter = pg.exporters.SVGExporter(self.fig.plotItem)
        else:
            exporter = pg.exporters.ImageExporter(self.fig.plotItem)
            exporter.params.param("width").setValue(
                int(res[0]), blockSignal=exporter.widthChanged
            )
            exporter.params.param("height").setValue(
                int(res[1]), blockSignal=exporter.heightChanged
            )
        exporter.export(fname)

    def cp2clipboard(self):
        img = self.widget.grab()
        QApplication.clipboard().setImage(img.toImage())

    #  def printing(self):
    #  exporter = pg.exporters.PrintExporter(self.fig.plotItem)
    #  exporter.export()


class _polar_layout_plot:
    def __init__(self, layout, widget, data):
        self.layout = layout
        self.widget = widget
        self.data = data
        self.ra = 1.3  # outer radius of the plot (because autoscale fails)
        self.r_slottext = 1.2  # radius slot number
        self.r_cs = [1, 0.9]  # radius coil sides (first and second layer)
        self.r_magnets = 0.75  # radius of the magnets

        if self.layout is None:
            self.app = pg.mkQApp()
            self.fig = pg.PlotWidget()  # title='xyz'
        else:
            self.fig = pg.PlotWidget()
            self.layout.addWidget(self.fig)
        self.fig.setAspectLocked(lock=True, ratio=1)
        self.fig.getAxis("bottom").hide()
        self.fig.getAxis("left").hide()
        self.leg = self.fig.addLegend(offset=(-10, 10))

    def plot(self, data=None, show=False, optimize_overhang=False, draw_poles=False):
        self.show = show
        if data is not None:
            self.data = data

        S = self.data.get_phases()
        Q = self.data.get_num_slots()
        P = self.data.get_num_polepairs() * 2
        w = self.data.get_windingstep()
        num_layers = self.data.get_num_layers()

        self.fig.clear()
        _pg_clear_legend(self.leg)

        self.fig.disableAutoRange()  # disable because of porformance
        # (a lot of elements are plottet)

        ovh = analyse.create_wdg_overhang(S, Q, num_layers)
        if optimize_overhang:
            head = ovh.get_overhang(wstep=None)
        else:
            head = ovh.get_overhang(wstep=w)

        def get_pos(num, r=1):
            """
            returns position (x, y) of the coil side for the given
            slot number
            """
            alpha = 2 * np.pi / Q * num
            vec = r * np.exp(1j * alpha)
            return vec.real, vec.imag

        # plot slot number
        for k in range(Q):
            text = pg.TextItem(anchor=(0.5, 0.5))
            text.setPlainText(str(k + 1))
            text.setPos(*get_pos(k, r=self.r_slottext))
            text.setColor("k")
            if Q > 60:
                text.setScale(1 / (Q / 60))
            self.fig.addItem(
                text, ignoreBounds=True
            )  # ignore because autoRange have problems with it

        # plot coil side with winding direction
        for km in range(len(S)):
            for kl in range(len(S[km])):
                layer = S[km][kl]
                for cs in layer:
                    text = pg.TextItem(anchor=(0.5, 0.5))
                    if cs > 0:
                        text.setHtml("&otimes;")
                    else:
                        text.setHtml("&#8857;")  # oint
                    text.setPos(*get_pos(abs(cs) - 1, r=self.r_cs[kl]))
                    text.setColor(get_phase_color(km))
                    if Q <= 60:
                        text.setScale(2)
                    else:
                        text.setScale(2 / (Q / 60))
                    self.fig.addItem(
                        text, ignoreBounds=True
                    )  # ignore because autoRange have problems with it

        # plot connections (winding overhang)
        for km in range(len(head)):
            x, y = [], []
            for line in head[km]:
                cs1_layer = line[4][0]
                cs2_layer = line[4][1]
                x1, y1 = get_pos(line[0] - 1, r=self.r_cs[cs1_layer])
                x2, y2 = get_pos(line[1] - 1, r=self.r_cs[cs2_layer])
                x += [x1, x2, np.nan]
                y += [y1, y2, np.nan]
            pen = pg.mkPen(get_phase_color(km), width=1.5)

            # Bug in pyqtgraph / PyQt5 > v5.12.1
            #  curve = pg.PlotCurveItem(
            #  x, y, pen=pen, name="Phase " + str(km + 1), connect="finite"
            #  )
            #  self.fig.addItem(curve)

            i_name = 0
            for xtmp, ytmp in zip(*group_on_nan(x, y)):
                name = "Phase " + str(km + 1) if i_name == 0 else None
                curve = pg.PlotCurveItem(
                    xtmp, ytmp, pen=pen, name=name, connect="finite"
                )
                self.fig.addItem(curve)
                i_name += 1

        # draw poles
        if draw_poles:
            alpha_m = config["plt"]["magnet_alpha_m"]
            magnet_colors = config["plt"]["magnet_colors"]
            magnet_linewidth = config["plt"]["magnet_linewidth"]
            alpha = np.linspace(0, 2 * np.pi / P * alpha_m, 100)
            alpha -= np.max(alpha / 2)  # middle of the pole on alpha = 0°
            for kp in range(P):
                vec = self.r_magnets * np.exp(1j * (alpha + kp * 2 * np.pi / P))
                if kp % 2 == 0:
                    pen = pg.mkPen(magnet_colors[0], width=magnet_linewidth)
                else:
                    pen = pg.mkPen(magnet_colors[1], width=magnet_linewidth)
                curve = pg.PlotCurveItem(vec.real, vec.imag, pen=pen)
                self.fig.addItem(curve)

        #  self.fig.autoRange()  # fails -> manual range
        self.fig.setXRange(-self.ra, self.ra)
        self.fig.setYRange(-self.ra, self.ra)
        if self.show:
            self.fig.show()
            self.app.exec_()

    def save(self, fname, res):
        if self.layout == None:
            self.app.processEvents()
        if os.path.splitext(fname)[-1].upper() == ".SVG":
            exporter = pg.exporters.SVGExporter(self.fig.plotItem)
        else:
            exporter = pg.exporters.ImageExporter(self.fig.plotItem)
            exporter.params.param("width").setValue(
                int(res[0]), blockSignal=exporter.widthChanged
            )
            exporter.params.param("height").setValue(
                int(res[1]), blockSignal=exporter.heightChanged
            )
        exporter.export(fname)

    def cp2clipboard(self):
        img = self.fig.grab()
        QApplication.clipboard().setImage(img.toImage())


class _overhang_plot:
    bz = 0.5  # tooth width
    hz = 0.5  # slot height
    h1 = 0.6  # height of the coil side
    #  h2 = 1.5  # height of the winding overhang
    db1 = 0.1  # distance between coil side and slot center
    Np1 = 101  # number of plotting points in the winding overhang per coil

    def __init__(self, layout, widget, data):
        self.layout = layout
        self.widget = widget
        self.data = data
        self.Q = 0
        self.slot = {}
        self.devide = "h"  # slot devide h (horizontal) or v (vertical

        if self.layout is None:
            self.app = pg.mkQApp()
            self.fig = pg.PlotWidget()  # title='xyz'
        else:
            self.fig = pg.PlotWidget()
            self.layout.addWidget(self.fig)
        #  self.fig.setAspectLocked(lock=True, ratio=1)
        self.fig.getAxis("bottom").hide()
        self.fig.getAxis("left").hide()
        self.leg = self.fig.addLegend(offset=(-10, 10))

    def plot(self, data=None, show=False, optimize_overhang=False):
        self.show = show
        if data is not None:
            self.data = data

        S = self.data.get_phases()
        Q = self.data.get_num_slots()
        w = self.data.get_windingstep()
        num_layers = self.data.get_num_layers()

        self.fig.clear()
        _pg_clear_legend(self.leg)

        self.fig.disableAutoRange()  # disable because of porformance
        # (a lot of elements are plottet)

        # plot slots - lines
        #  x, y = gen_slot_lines(Q, bz = self.bz, hz = self.hz)
        #  pen = pg.mkPen(color='#808080', width = 1.5)
        #  curve = pg.PlotCurveItem(x, y, pen=pen, connect='finite')
        #  self.fig.addItem(curve)

        # plot slots - filling
        brush = pg.mkBrush(color="#BFBFBF")
        x, y_upper, y_lower = gen_slot_filling(Q, bz=self.bz, hz=self.hz)

        # Bug in pyqtgraph / PyQt5 > v5.12.1
        #  c1 = pg.PlotCurveItem(x, y_upper, connect="finite")
        #  c2 = pg.PlotCurveItem(x, y_lower, connect="finite")
        #  fill = pg.FillBetweenItem(c1, c2, brush=brush)
        #  self.fig.addItem(fill)

        _, y_upper = group_on_nan(x, y_upper)
        x, y_lower = group_on_nan(x, y_lower)
        for xtmp, y_upper_tmp, y_lower_tmp in zip(x, y_upper, y_lower):
            c1 = pg.PlotCurveItem(xtmp, y_upper_tmp, connect="finite")
            c2 = pg.PlotCurveItem(xtmp, y_lower_tmp, connect="finite")
            fill = pg.FillBetweenItem(c1, c2, brush=brush)
            self.fig.addItem(fill)

        ovh = analyse.create_wdg_overhang(S, Q, num_layers)
        if optimize_overhang:
            head = ovh.get_overhang(wstep=None)
        else:
            head = ovh.get_overhang(wstep=w)

        i = 1
        for phase in head:
            x = []
            y = []
            for coil in phase:
                w = coil[2]
                x_, y_ = gen_coil_lines(
                    coil[2], h1=self.h1, h2=0.5 + w / 6, db1=self.db1, Np1=self.Np1
                )
                x_, y_ = np.array(x_), np.array(y_)

                direct = coil[3]
                if direct > 0:
                    x_ += coil[0] - 1
                else:
                    x_ += coil[1] - 1

                # split coil on right border
                x_2 = x_.copy()
                y_2 = y_.copy()
                x_2[x_ < Q - 0.5] = np.nan
                x_2 -= Q
                y_2[x_ < Q - 0.5] = np.nan
                y_[x_ > Q - 0.5] = np.nan
                x_[x_ > Q - 0.5] = np.nan
                x += x_.tolist()
                y += y_.tolist()
                x += [np.nan]
                y += [np.nan]
                x += x_2.tolist()
                y += y_2.tolist()
                x += [np.nan]
                y += [np.nan]

                # plot arrows (winding direction)
                arrow = pg.ArrowItem(
                    angle=90,
                    headLen=16,
                    pen={"color": get_phase_color(i - 1)},
                    brush=get_phase_color(i - 1),
                )
                if coil[3] > 0:
                    arrow.setPos(coil[0] - 1 + self.db1, 0)
                else:
                    arrow.setPos(coil[0] - 1 - self.db1, 0)
                self.fig.addItem(arrow)

                arrow = pg.ArrowItem(
                    angle=-90,
                    headLen=16,
                    pen={"color": get_phase_color(i - 1)},
                    brush=get_phase_color(i - 1),
                )
                if coil[3] > 0:
                    arrow.setPos(coil[1] - 1 - self.db1, 0)
                else:
                    arrow.setPos(coil[1] - 1 + self.db1, 0)
                self.fig.addItem(arrow)

            pen = pg.mkPen(color=get_phase_color(i - 1), width=config["plt"]["lw"])

            # Bug in pyqtgraph / PyQt5 > v5.12.1
            #  curve = pg.PlotCurveItem(
            #  x, y, pen=pen, connect="finite", name="Phase " + str(i)
            #  )
            #  self.fig.addItem(curve)

            i_name = 0
            for xtmp, ytmp in zip(*group_on_nan(x, y)):
                name = "Phase " + str(i) if i_name == 0 else None
                curve = pg.PlotCurveItem(
                    xtmp, ytmp, pen=pen, connect="finite", name=name
                )
                self.fig.addItem(curve)
                i_name += 1

            i += 1

        # plot slot number
        for k in range(Q):
            text = pg.TextItem(anchor=(0.5, 0.5))
            text.setPlainText(str(k + 1))
            text.setPos(k, -self.h1 / 2)
            text.setColor("k")
            self.fig.addItem(
                text, ignoreBounds=True
            )  # ignore because autoRange have problems with it

        self.fig.autoRange()
        if self.show:
            self.fig.show()
            self.app.exec_()

    def save(self, fname, res):
        if self.layout == None:
            self.app.processEvents()
        if os.path.splitext(fname)[-1].upper() == ".SVG":
            exporter = pg.exporters.SVGExporter(self.fig.plotItem)
        else:
            exporter = pg.exporters.ImageExporter(self.fig.plotItem)
            exporter.params.param("width").setValue(
                int(res[0]), blockSignal=exporter.widthChanged
            )
            exporter.params.param("height").setValue(
                int(res[1]), blockSignal=exporter.heightChanged
            )
        exporter.export(fname)

    def cp2clipboard(self):
        img = self.fig.grab()
        QApplication.clipboard().setImage(img.toImage())


class _slot_star:
    def __init__(self, layout, widget, data, table):
        self.layout = layout
        self.widget = widget
        self.data = data
        self.table = table

        if self.layout is None:
            self.app = pg.mkQApp()
            self.fig = pg.PlotWidget()  # title='xyz'
        else:
            self.fig = pg.PlotWidget()
            self.layout.addWidget(self.fig)
        self.fig.getAxis("bottom").hide()
        self.fig.getAxis("left").hide()
        self.fig.setAspectLocked(lock=True, ratio=1)
        self.leg = self.fig.addLegend(offset=(-10, 10))

    def plot(self, data, harmonic_idx=0, ForceX=None, show=False):
        self.data = data
        self.show = show
        if harmonic_idx < 0:
            harmonic_idx = 0

        self.fig.clear()
        _pg_clear_legend(self.leg)

        self.fig.disableAutoRange()  # disable because of porformance
        # (a lot of elements are plottet)

        xlim = [
            0,
            0,
        ]  # find limit manually, because autoscale doesn't work with plt.arrow()
        ylim = [0, 0]

        def plot_vek(vek, color="k", dangle=None, idx=None):
            """
            vek: list of complex values
            dangle: All phasors get shifted by dangle
            """
            if dangle is not None:
                A = np.abs(vek)
                ph = np.angle(vek)
                ph -= dangle
                vek = A * np.exp(1j * ph)

            old = 0 + 0j
            v = np.cumsum(vek)

            for k in range(
                len(vek)
            ):  # find limit manually, because autoscale doesn't work with plt.arrow()
                if min(v.real) < xlim[0]:
                    xlim[0] = min(v.real)
                if max(v.real) > xlim[1]:
                    xlim[1] = max(v.real)
                if min(v.imag) < ylim[0]:
                    ylim[0] = min(v.imag)
                if max(v.imag) > ylim[1]:
                    ylim[1] = max(v.imag)

                arrow = pg.ArrowItem(
                    angle=180 - np.angle(vek[k]) / np.pi * 180,
                    headLen=12,
                    pen={"color": color},
                    brush=color,
                )
                arrow.setPos(old.real + vek[k].real, old.imag + vek[k].imag)
                self.fig.addItem(arrow)
                line = pg.PlotCurveItem(
                    [old.real, vek[k].real + old.real],
                    [old.imag, vek[k].imag + old.imag],
                    pen={"color": color, "width": config["plt"]["lw"]},
                )
                self.fig.addItem(line)

                old = v[k]
            line = pg.PlotCurveItem(
                [0.0, v[-1].real],
                [0.0, v[-1].imag],
                name="Phase " + str(idx + 1),
                pen={
                    "color": color,
                    "width": config["plt"]["lw"],
                    "style": pg.QtCore.Qt.DashLine,
                },
            )
            self.fig.addItem(line)

        if ForceX:
            angle_phase1 = np.angle(np.sum(self.data.results["Ei_el"][harmonic_idx][0]))
        else:
            angle_phase1 = None
        for k in range(len(self.data.results["Ei_el"][harmonic_idx])):
            plot_vek(
                self.data.results["Ei_el"][harmonic_idx][k],
                color=get_phase_color(k),
                dangle=angle_phase1,
                idx=k,
            )

        self.fig.autoRange()
        if show:
            self.fig.show()
            self.app.exec_()

        # update table:
        ei = self.data.results["Ei_el"][harmonic_idx]

        if self.table is not None:
            self.table.setRowCount(len(ei))
            self.table.setColumnCount(3)

            for km in range(len(ei)):
                A = np.abs(np.sum(ei[km]))
                ph = np.angle(np.sum(ei[km])) / np.pi * 180
                self.table.setItem(km, 0, QTableWidgetItem(str(km + 1)))
                self.table.setItem(km, 1, QTableWidgetItem(str(round(A, 1))))
                self.table.setItem(km, 2, QTableWidgetItem(str(round(ph, 1))))

                for i in range(3):  # set all cells as not editable
                    self.table.item(km, i).setFlags(
                        QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled
                    )

            for k1 in range(len(ei)):
                self.table.resizeColumnToContents(k1)

            self.table.setHorizontalHeaderLabels(["m", "A", "phase"])
            #  self.table.setVerticalHeaderLabels(['']*np.shape(kw)[0])
            afont = QFont()
            afont.setBold(True)

            for k in range(3):
                self.table.horizontalHeaderItem(k).setFont(afont)

    def save(self, fname, res):
        if self.layout == None:
            self.app.processEvents()
        if os.path.splitext(fname)[-1].upper() == ".SVG":
            exporter = pg.exporters.SVGExporter(self.fig.plotItem)
        else:
            exporter = pg.exporters.ImageExporter(self.fig.plotItem)
            exporter.params.param("width").setValue(
                int(res[0]), blockSignal=exporter.widthChanged
            )
            exporter.params.param("height").setValue(
                int(res[1]), blockSignal=exporter.heightChanged
            )
        exporter.export(fname)

    #  def printing(self):
    #  exporter = pg.exporters.PrintExporter(self.fig.plotItem)
    #  exporter.export()

    def cp2clipboard(self):
        img = self.fig.grab()
        QApplication.clipboard().setImage(img.toImage())


class _windingfactor:
    def __init__(self, layout, widget, data, table):
        self.layout = layout
        self.widget = widget
        self.data = data
        self.table = table

        if self.layout is None:
            self.app = pg.mkQApp()
            self.fig = pg.PlotWidget()  # title='xyz'
        else:
            self.fig = pg.PlotWidget()
            self.layout.addWidget(self.fig)
        self.fig.setLabel("bottom", "<div>ordinal number &nu;</div>")
        self.fig.setLabel(
            "left", "<div>Windingfactor kw</div>"
        )  # <div>Windingfactor &xi;</div>
        self.fig.showGrid(x=True, y=True)
        self.leg = self.fig.addLegend(offset=(-10, 10))
        self.fig.setLimits(xMin=0, yMin=0)

    def plot(self, data, mechanical=False, show=False):
        self.data = data
        self.show = show
        self.fig.clear()
        _pg_clear_legend(self.leg)

        if mechanical:
            nu = np.array(self.data.results["nu_mech"])
            kw = np.array(self.data.results["kw_mech"])
        else:
            nu = np.array(self.data.results["nu_el"])
            kw = np.array(self.data.results["kw_el"])

        #  gap = 0.2
        N = np.shape(kw)[1]
        for k in range(N):
            dx = nu[1] - nu[0]
            x = nu - dx / N / 1.5 + dx / N / 1.5 * k

            pen = pg.mkPen(width=0.0, color=get_phase_color(k))
            bar = pg.BarGraphItem(
                x=x,
                height=np.abs(kw[:, k]),
                width=(dx) / 1.5 / N,
                brush=get_phase_color(k),
                pen=pen,
            )  # , name='Phase '+str(k+1)
            self.fig.addItem(bar)

            # workaround: Plot points for legend because 'BarGraphItem' has no 'name'
            line = pg.PlotCurveItem(
                x, np.abs(kw[:, k]), ".", name="Phase " + str(k + 1), pen=pen
            )
            self.fig.addItem(line)

        if show:
            self.fig.show()
            self.app.exec_()

        if self.table is not None:
            # update table:
            self.table.setRowCount(np.shape(kw)[0])
            self.table.setColumnCount(N + 1)

            for k1 in range(np.shape(kw)[0]):
                self.table.setItem(k1, 0, QTableWidgetItem(str(nu[k1])))
                for k2 in range(np.shape(kw)[1]):
                    self.table.setItem(
                        k1, k2 + 1, QTableWidgetItem(str(round(kw[k1, k2], 3)))
                    )
                    # set all cells as not editable
                    self.table.item(k1, k2 + 1).setFlags(
                        QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled
                    )
            for k1 in range(np.shape(kw)[0]):
                self.table.resizeColumnToContents(k1)

            self.table.setHorizontalHeaderLabels(["nu"] + self.data.get_phasenames())
            self.table.setVerticalHeaderLabels([""] * np.shape(kw)[0])
            afont = QFont()
            afont.setBold(True)

            for k in range(self.data.get_num_phases() + 1):
                self.table.horizontalHeaderItem(k).setFont(afont)

    def save(self, fname, res):
        if self.layout == None:
            self.app.processEvents()
        if os.path.splitext(fname)[-1].upper() == ".SVG":
            exporter = pg.exporters.SVGExporter(self.fig.plotItem)
        else:
            exporter = pg.exporters.ImageExporter(self.fig.plotItem)
            exporter.params.param("width").setValue(
                int(res[0]), blockSignal=exporter.widthChanged
            )
            exporter.params.param("height").setValue(
                int(res[1]), blockSignal=exporter.heightChanged
            )
        exporter.export(fname)

    #  def printing(self):
    #  exporter = pg.exporters.PrintExporter(self.fig.plotItem)
    #  exporter.export()

    def cp2clipboard(self):
        img = self.fig.grab()
        QApplication.clipboard().setImage(img.toImage())


class _mmk:
    def __init__(self, layout, widget, data, table):
        self.layout = layout
        self.widget = widget
        self.data = data
        self.table = table

        if self.layout is None:
            self.app = pg.mkQApp()

        self.l = pg.GraphicsLayoutWidget()
        self.fig1 = self.l.addPlot(row=1, col=1, rowspan=1)
        self.fig2 = self.l.addPlot(row=2, col=1, rowspan=1)
        if self.layout is not None:
            self.layout.addWidget(self.l)

        self.fig1.setLabel("bottom", "circumferential stator slots")
        self.fig1.setLabel("left", "MMF in A")
        self.fig1.showGrid(x=True, y=True)
        self.leg1 = self.fig1.addLegend(offset=(-10, 10))

        self.fig2.setLabel("bottom", "circumferential stator slots")
        self.fig2.setLabel("left", "Current in slot in A")
        self.fig2.showGrid(x=True, y=True)

    def plot(self, data, phase=0, small_update=False, show=False):
        """
        Plottet MMF-Kurve

        Parameters
        ----------
        plot_MMK_nu :           iterable - list/tuple
                                Liste/Tuple mit Ordnungszahlen, die geplottet werden sollen
        plot_MMK_greater_than : scalar, float
                                Plotte alle Oberschwingungen mit der Amplitude größer als
        """
        self.data = data
        self.show = show
        plot_MMK_greater_than = config["plot_MMF_harmonics"]

        phi, MMK, theta = analyse.calc_MMK(
            self.data.get_num_slots(),
            self.data.get_num_phases(),
            self.data.get_phases(),
            self.data.get_turns(),
            angle=phase,
        )
        phi = np.array(phi)
        threshold = config["threshold_MMF_harmonics"]
        nu, A, phase = self.data.get_MMF_harmonics(threshold=threshold)

        self.fig1.clear()
        self.fig1.disableAutoRange()  # disable because of porformance
        # (a lot of elements are plottet)
        _pg_clear_legend(self.leg1)

        pen = pg.mkPen(color=get_line_color(0), width=config["plt"]["lw"])
        curve = pg.PlotCurveItem(phi, MMK, pen=pen)
        self.fig1.addItem(curve)

        # Plotte Oberschwingungen
        i = 0
        for n, a, p in zip(nu, A, phase):
            if 1 / max(A) * a > plot_MMK_greater_than:
                pen = pg.mkPen(
                    color=get_line_color(i + 1), width=config["plt"]["lw_thin"]
                )
                curve = pg.PlotCurveItem(
                    phi,
                    a * np.cos(n * phi / phi[-1] * 2 * np.pi + p),
                    name="<div>&nu;={}</div>".format(n),
                    pen=pen,
                )
                self.fig1.addItem(curve)
                i += 1
        self.fig1.autoRange()
        self.fig1.setLimits(xMin=min(phi), xMax=max(phi))

        self.fig2.clear()
        pen = pg.mkPen(width=0.0, color=get_line_color(0))
        bar = pg.BarGraphItem(
            x=range(len(theta)),
            height=theta,
            width=0.5,
            brush=get_line_color(0),
            pen=pen,
        )  # , name='Phase '+str(k+1)
        self.fig2.addItem(bar)
        self.fig2.setXRange(min(phi), max(phi))
        if show:
            self.l.show()
            self.app.exec_()

        if self.table is not None:
            # update table:
            self.table.setRowCount(len(A))
            self.table.setColumnCount(3)
            self.table.setHorizontalHeaderLabels(["nu", "Amp", "[%]"])
            Arel = 100 / np.max(A) * A
            for k1 in range(len(A)):
                self.table.setItem(k1, 0, QTableWidgetItem(str(nu[k1])))
                self.table.setItem(k1, 1, QTableWidgetItem(str(round(A[k1], 3))))
                self.table.setItem(k1, 2, QTableWidgetItem(str(round(Arel[k1], 1))))
                for i in range(3):  # set all cells as not editable
                    self.table.item(k1, i).setFlags(
                        QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled
                    )
            for k1 in range(3):
                self.table.resizeColumnToContents(k1)

    def save(self, fname, res):
        if self.layout == None:
            self.app.processEvents()

        if os.path.splitext(fname)[-1].upper() == ".SVG":
            exporter = pg.exporters.SVGExporter(self.l.ci)
        else:
            exporter = pg.exporters.ImageExporter(self.l.ci)
            exporter.params.param("width").setValue(
                int(res[0]), blockSignal=exporter.widthChanged
            )
            exporter.params.param("height").setValue(
                int(res[1]), blockSignal=exporter.heightChanged
            )
        exporter.export(fname)

    #  def printing(self):
    #  exporter = pg.exporters.PrintExporter(self.l.ci)
    #  exporter.export()

    def cp2clipboard(self):
        img = self.l.grab()
        QApplication.clipboard().setImage(img.toImage())


class _Report_Highlighter(QSyntaxHighlighter):
    """
    Syntax highlighting for report
    """

    def __init__(self, parent=None):
        super(_Report_Highlighter, self).__init__(parent)
        self.highlightingRules = []

        # headings
        Format = QTextCharFormat()
        Format.setForeground(QColor("#3536A9"))
        Format.setFontWeight(QFont.Bold)
        self.highlightingRules.append(
            (QtCore.QRegExp(r"={2,}"), Format)
        )  # minumum two '='
        self.highlightingRules.append(
            (QtCore.QRegExp(r"[A-Z]+\s"), Format)
        )  # word in upper letters with following whitespace
        self.highlightingRules.append(
            (QtCore.QRegExp(r"([A-Z]+)$"), Format)
        )  # last word in upper letters
        # Variables
        Format = QTextCharFormat()
        Format.setFontItalic(True)
        self.highlightingRules.append((QtCore.QRegExp(r"[A-Za-z0-9_-]+\:\s"), Format))
        # numbers
        Format = QTextCharFormat()
        Format.setForeground(QColor("#008000"))
        self.highlightingRules.append(
            (
                QtCore.QRegExp(r"(^|\s|-|/)[-+]?[0-9]*\.?[0-9]+([eE][-+]?[0-9]+)?"),
                Format,
            )
        )
        # comments in brackets
        Format = QTextCharFormat()
        Format.setForeground(QColor("#6C6C6C"))
        self.highlightingRules.append((QtCore.QRegExp(r"\(([^)]+)\)"), Format))

    def highlightBlock(self, text):
        for pattern, format in self.highlightingRules:
            expression = QtCore.QRegExp(pattern)
            index = expression.indexIn(text)
            while index >= 0:
                length = expression.matchedLength()
                self.setFormat(index, length, format)
                index = expression.indexIn(text, index + length)


class _report:
    def __init__(self, reportEdit):
        self.search = ""
        self.search_idx = -1
        self.reportEdit = reportEdit
        font = QFont(config["report_txt"]["font"], config["report_txt"]["fontsize"])
        font.setStyleHint(QFont.TypeWriter)
        self.reportEdit.setCurrentFont(font)

    def find(self, search):
        """
        Finding and selection of strings in the report
        """

        def reset_curser():
            c = self.reportEdit.textCursor()
            c.setPosition(0)
            self.reportEdit.setTextCursor(c)

        def select(start, end):
            cursor = self.reportEdit.textCursor()
            cursor.setPosition(start)
            cursor.setPosition(end, QTextCursor.KeepAnchor)
            self.reportEdit.setTextCursor(cursor)

        if not search:
            reset_curser()
            return 0

        if search != self.search:
            self.search = search
            text = self.reportEdit.toPlainText()
            self.found = []

            p = re.compile(search, flags=re.IGNORECASE)
            for p in p.finditer(text):
                self.found.append(p)
            if self.found:
                self.search_idx = -1
            else:
                self.search_idx = None
                reset_curser()

        if self.found:
            self.search_idx += 1
            if self.search_idx > len(self.found) - 1:
                self.search_idx = 0
            select(
                self.found[self.search_idx].start(), self.found[self.search_idx].end()
            )
            return len(self.found)
        else:
            return -1

    def cp2clipboard(self):
        QApplication.clipboard().setText(self.reportEdit.toPlainText())



from PyQt5 import uic
from PyQt5 import QtGui
from PyQt5.QtWidgets import QTreeWidgetItem, QDialog, QFileDialog
from swat_em import datamodel, project
import os
import sys

if getattr(sys, "frozen", False) and hasattr(sys, "_MEIPASS"):
    __dir__ = sys._MEIPASS  # for pyinstaller
else:
    __dir__ = os.path.dirname(os.path.abspath(__file__))


class import_winding(QDialog):
    """
    Dialog for importing windings from a file
    """

    def __init__(self):
        super().__init__()
        uic.loadUi(os.path.join(__dir__, "ui", "ImportWinding.ui"), self)
        self.ButtonCheckAll.clicked.connect(self.check_all)
        self.ButtonCheckNone.clicked.connect(self.check_none)

    def create_list(self, filename):

        self.models = project()
        self.models.load_from_file(filename)

        self.items = []
        for k in range(self.models.get_num_models()):
            data = self.models.get_model_by_index(k)
            bc, txt = data.get_basic_characteristics()

            # title
            font = QtGui.QFont()
            font.setBold(True)

            self.items.append(QTreeWidgetItem([data.title]))
            self.items[-1].setFont(0, font)

            # Infos
            font = QtGui.QFont()
            font.setItalic(True)

            child = QTreeWidgetItem(
                [
                    "Q: {}, 2p: {}, m: {}".format(
                        data.get_num_slots(),
                        2 * data.get_num_polepairs(),
                        data.get_num_phases(),
                    )
                ]
            )
            child.setFont(0, font)
            self.items[-1].addChild(child)

            l = [str(round(k, 3)) for k in bc["kw1"]]

            child = QTreeWidgetItem(["Winding factor kw1: {}".format(", ".join(l))])
            child.setFont(0, font)
            self.items[-1].addChild(child)

            child = QTreeWidgetItem(["Notes: " + data.get_notes()])
            child.setFont(0, font)
            self.items[-1].addChild(child)

            self.items[-1].setCheckState(0, False)
            self.treeWidget.addTopLevelItem(self.items[-1])

        self.treeWidget.expandToDepth(0)

    def check_all(self):
        for item in self.items:
            item.setCheckState(0, True)

    def check_none(self):
        for item in self.items:
            item.setCheckState(0, False)

    def run(self):
        options = QFileDialog.Options()
        filename, _ = QFileDialog.getOpenFileName(
            self, "Open winding file", "", "Winding Files (*.wdg)", options=options
        )
        if not filename:
            return None
        self.create_list(filename)
        ok = self.exec_()
        if ok:
            data = []
            for i, item in enumerate(self.items):
                if item.checkState(0):
                    data.append(self.models.get_model_by_index(i))
            return data
        else:
            return None



# -*- coding: utf-8 -*-

import os
import sys
sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

from swat_em.wdggenerator import genwdg
from swat_em.datamodel import datamodel
from swat_em import wdggenerator
import numpy as np



#  data = datamodel()
#  data.genwdg(Q = 12, P = 10, m = 3, layers = 2)


#  Q = 12
#  p = 5
#  m = 3
#  wstep = 1
#  layers = 1
#  ret = genwdg(Q, 2*p, m, wstep, layers) # 12/10
#  wdglayout = ret['phases']
#  data = datamodel()
#  data.set_machinedata(Q = Q, p = p, m = m)
#  data.set_phases(wdglayout)
#  data.get_basic_characteristics()
#  data.analyse_wdg()

# 0.013 sec






data = []
Qlist = [3, 6, 9, 12, 15, 18, 21] 
Plist = [2, 4, 6, 8, 10, 12, 14, 16]
m = 3
wstep = -1
layers = 2

for iQ, kQ in enumerate(Qlist):
    data.append([])
    for iP, kP in enumerate(Plist):
        d = datamodel()
        d.set_machinedata(Q = kQ, m = m, p = kP/2)
        ret = wdggenerator.genwdg(kQ, kP, m, wstep, layers)
        d.set_phases(S = ret['phases'], wstep = ret['wstep'])
        d.set_valid(ret['valid'], ret['error'])
        bc, bc_str = d.get_basic_characteristics()
        data[iQ].append(d)

# 0.261 sec
# 0.226 sec # durch MMK.tolist() entfernen
# 0.206 sec # durch np.max statt max
# 0.106 sec # double linked linkage vektorisiert
# 0.093 sec # np.roll -> deque.rotate
# 0.086 sec # np.isclose -> math.isclose weil nur Skalare







#data = []
#Qlist = [3, 6, 9, 12, 15, 18, 21] 
#Plist = [2, 4, 6, 8, 10, 12, 14, 16]
#m = 3
#wstep = -1
#layers = 2
#
#for iQ, kQ in enumerate(Qlist):
#    data.append([])
#    for iP, kP in enumerate(Plist):
#        d = datamodel()
#        d.set_machinedata(Q = kQ, m = m, p = kP/2)
#        ret = wdggenerator.genwdg(kQ, kP, m, wstep, layers)
#        d.set_phases(S = ret['phases'], wstep = ret['wstep'])
#        d.set_valid(ret['valid'], ret['error'])
#        d.analyse_wdg()
#        bc, bc_str = d.get_basic_characteristics()
#        data[iQ].append(d)

# 6.5




"""
def test2():
    print('Test single layer toothcoilwinding')
    Q = 18
    p = 8
    m = 3
    wstep = 1
    layers = 1
    #  U = [[1,-2,3,-4,-11,12]]
    #  V = [[-5,6,13,-14,15,-16]]
    #  W = [[7,-8,9,-10,-17,18]]
    U = [[1, -2, -9, 10, 17, -18],[]]
    V = [[-3, 4, 11, -12, 13, -14],[]]
    W = [[5, -6, 7, -8, -15, 16],[]]
   
    kw1 = 0.9452		
    kw5 = -0.1398		
    kw7 = -0.0607
    ret = genwdg(Q, 2*p, m, wstep, layers) # 12/10
    wdglayout = ret['phases']
    
    print(wdglayout)
    assert [U, V, W] == wdglayout
    
    data = datamodel()
    #  data.set_config(get_init_config())
    data.set_machinedata(Q = Q, p = p, m = m)
    data.set_phases(wdglayout)
    data.analyse_wdg()

    idx = data.results['nu_el'].index(1)
    assert kw1 == np.round(data.results['kw_el'][idx][0], 4) # phase U
    assert kw1 == np.round(data.results['kw_el'][idx][1], 4) # phase V
    assert kw1 == np.round(data.results['kw_el'][idx][2], 4) # phase W
    idx = data.results['nu_el'].index(5)
    assert kw5 == np.round(data.results['kw_el'][idx][0], 4) # phase U
    assert kw5 == np.round(data.results['kw_el'][idx][1], 4) # phase V
    assert kw5 == np.round(data.results['kw_el'][idx][2], 4) # phase W
    idx = data.results['nu_el'].index(7)
    assert kw7 == np.round(data.results['kw_el'][idx][0], 4) # phase U
    assert kw7 == np.round(data.results['kw_el'][idx][1], 4) # phase V
    assert kw7 == np.round(data.results['kw_el'][idx][2], 4) # phase W
    

def test3():
    print('Test single layer toothcoilwinding')
    Q = 12
    p = 4
    m = 3
    wstep = 1
    layers = 1
    U = [[1,-2,7,-8],[]]
    V = [[5,-6,11,-12],[]]
    W = [[3,-4,9,-10],[]]
    kw1 = 0.866
    kw5 = -0.866
    kw7 = 0.866
    ret = genwdg(Q, 2*p, m, wstep, layers) # 12/10
    wdglayout = ret['phases']
    print(wdglayout)
    assert [U, V, W] == wdglayout

    data = datamodel()
    data.set_machinedata(Q = Q, p = p, m = m)
    data.set_phases(wdglayout)
    data.analyse_wdg()

    idx = data.results['nu_el'].index(1)
    assert kw1 == np.round(data.results['kw_el'][idx][0], 4) # phase U
    assert kw1 == np.round(data.results['kw_el'][idx][1], 4) # phase V
    assert kw1 == np.round(data.results['kw_el'][idx][2], 4) # phase W
    idx = data.results['nu_el'].index(5)
    assert kw5 == np.round(data.results['kw_el'][idx][0], 4) # phase U
    assert kw5 == np.round(data.results['kw_el'][idx][1], 4) # phase V
    assert kw5 == np.round(data.results['kw_el'][idx][2], 4) # phase W
    idx = data.results['nu_el'].index(7)
    assert kw7 == np.round(data.results['kw_el'][idx][0], 4) # phase U
    assert kw7 == np.round(data.results['kw_el'][idx][1], 4) # phase V
    assert kw7 == np.round(data.results['kw_el'][idx][2], 4) # phase W


def test4():
    print('Test single layer toothcoilwinding')
    Q = 12
    p = 5
    m = 3
    wstep = 1
    layers = 1
    U = [[1,-2,-7,8],[]]
    V = [[-3,4,9,-10],[]]
    W = [[5,-6,-11,12],[]]
    kw1 = 0.9659		
    kw5 = -0.2588
    kw7 = 0.2588
    ret = genwdg(Q, 2*p, m, wstep, layers) # 12/10
    wdglayout = ret['phases']
    print(wdglayout)
    assert [U, V, W] == wdglayout

    data = datamodel()
    data.set_machinedata(Q = Q, p = p, m = m)
    data.set_phases(wdglayout)
    data.analyse_wdg()

    idx = data.results['nu_el'].index(1)
    assert kw1 == np.round(data.results['kw_el'][idx][0], 4) # phase U
    assert kw1 == np.round(data.results['kw_el'][idx][1], 4) # phase V
    assert kw1 == np.round(data.results['kw_el'][idx][2], 4) # phase W
    idx = data.results['nu_el'].index(5)
    assert kw5 == np.round(data.results['kw_el'][idx][0], 4) # phase U
    assert kw5 == np.round(data.results['kw_el'][idx][1], 4) # phase V
    assert kw5 == np.round(data.results['kw_el'][idx][2], 4) # phase W
    idx = data.results['nu_el'].index(7)
    assert kw7 == np.round(data.results['kw_el'][idx][0], 4) # phase U
    assert kw7 == np.round(data.results['kw_el'][idx][1], 4) # phase V
    assert kw7 == np.round(data.results['kw_el'][idx][2], 4) # phase W


def test5():
    print('Test single layer toothcoilwinding')
    Q = 24
    p = 11
    m = 3
    wstep = 1
    layers = 1
    U = [[1, -2, -11, 12, -13, 14, 23, -24],[]]
    V = [[-3, 4, -5, 6, 15, -16, 17, -18],[]]
    W = [[7, -8, 9, -10, -19, 20, -21, 22],[]]
    kw1 = 0.9577	
    kw5 = -0.2053
    kw7 = -0.1576
    ret = genwdg(Q, 2*p, m, wstep, layers) # 12/10
    wdglayout = ret['phases']
    print(wdglayout)
    assert [U, V, W] == wdglayout

    data = datamodel()
    data.set_machinedata(Q = Q, p = p, m = m)
    data.set_phases(wdglayout)
    data.analyse_wdg()

    idx = data.results['nu_el'].index(1)
    assert kw1 == np.round(data.results['kw_el'][idx][0], 4) # phase U
    assert kw1 == np.round(data.results['kw_el'][idx][1], 4) # phase V
    assert kw1 == np.round(data.results['kw_el'][idx][2], 4) # phase W
    idx = data.results['nu_el'].index(5)
    assert kw5 == np.round(data.results['kw_el'][idx][0], 4) # phase U
    assert kw5 == np.round(data.results['kw_el'][idx][1], 4) # phase V
    assert kw5 == np.round(data.results['kw_el'][idx][2], 4) # phase W
    idx = data.results['nu_el'].index(7)
    assert kw7 == np.round(data.results['kw_el'][idx][0], 4) # phase U
    assert kw7 == np.round(data.results['kw_el'][idx][1], 4) # phase V
    assert kw7 == np.round(data.results['kw_el'][idx][2], 4) # phase W


def test6():
    print('Test single layer full pitch winding')
    Q = 24
    p = 2
    m = 3
    wstep = -1
    layers = 1
    U = [[1, 2, -7, -8, 13, 14, -19, -20],[]]
    V = [[5, 6, -11, -12, 17, 18, -23, -24],[]]
    W = [[-3, -4, 9, 10, -15, -16, 21, 22],[]]
    kw1 = 0.9659	
    kw5 = -0.2588
    kw7 = 0.2588
    #  wdglayout = genwdg(Q, 2*p, m, wstep, layers) # 12/10
    ret = genwdg(Q, 2*p, m, wstep, layers)
    wdglayout = ret['phases']
    print(wdglayout)
    #  pdb.set_trace()
    assert [U, V, W] == wdglayout

    data = datamodel()
    data.set_machinedata(Q = Q, p = p, m = m)
    data.set_phases(wdglayout)
    data.analyse_wdg()

    idx = data.results['nu_el'].index(1)
    assert kw1 == np.round(data.results['kw_el'][idx][0], 4) # phase U
    assert kw1 == np.round(data.results['kw_el'][idx][1], 4) # phase V
    assert kw1 == np.round(data.results['kw_el'][idx][2], 4) # phase W
    idx = data.results['nu_el'].index(5)
    assert kw5 == np.round(data.results['kw_el'][idx][0], 4) # phase U
    assert kw5 == np.round(data.results['kw_el'][idx][1], 4) # phase V
    assert kw5 == np.round(data.results['kw_el'][idx][2], 4) # phase W
    idx = data.results['nu_el'].index(7)
    assert kw7 == np.round(data.results['kw_el'][idx][0], 4) # phase U
    assert kw7 == np.round(data.results['kw_el'][idx][1], 4) # phase V
    assert kw7 == np.round(data.results['kw_el'][idx][2], 4) # phase W


def test7():
    print('Test two layer full pitch winding')
    Q = 24
    p = 2
    m = 3
    wstep = 5
    layers = 2
    U = [[1, 2, -7, -8, 13, 14, -19, -20], [-6, -7, 12, 13, -18, -19, 24, 1]]
    V = [[5, 6, -11, -12, 17, 18, -23, -24], [-10, -11, 16, 17, -22, -23, 4, 5]]
    W = [[-3, -4, 9, 10, -15, -16, 21, 22], [8, 9, -14, -15, 20, 21, -2, -3]]
    kw1 = 0.9330	
    kw5 = -0.0670
    kw7 = 0.0670
    ret = genwdg(Q, 2*p, m, wstep, layers) # 12/10
    wdglayout = ret['phases']
    #  wdglayout = overlapping_fractional_slot_dlayer(Q, 2*p, m, wstep)
    print(wdglayout)
    #  pdb.set_trace()
    assert [U, V, W] == wdglayout

    data = datamodel()
    data.set_machinedata(Q = Q, p = p, m = m)
    data.set_phases(wdglayout)
    data.analyse_wdg()

    idx = data.results['nu_el'].index(1)
    assert kw1 == np.round(data.results['kw_el'][idx][0], 4) # phase U
    assert kw1 == np.round(data.results['kw_el'][idx][1], 4) # phase V
    assert kw1 == np.round(data.results['kw_el'][idx][2], 4) # phase W
    idx = data.results['nu_el'].index(5)
    assert kw5 == np.round(data.results['kw_el'][idx][0], 4) # phase U
    assert kw5 == np.round(data.results['kw_el'][idx][1], 4) # phase V
    assert kw5 == np.round(data.results['kw_el'][idx][2], 4) # phase W
    idx = data.results['nu_el'].index(7)
    assert kw7 == np.round(data.results['kw_el'][idx][0], 4) # phase U
    assert kw7 == np.round(data.results['kw_el'][idx][1], 4) # phase V
    assert kw7 == np.round(data.results['kw_el'][idx][2], 4) # phase W





"""



"""
# Test is maybe not correct!!!
def test8():
    print('Test overlapping fractional slot winding with single layer')
    # TODO: Referenzdaten ergänzen
    Q = 33
    p = 2
    m = 3
    #  wstep = 1
    layers = 1
    U = [[1, 2, 3, -9, -10, -11, 17, 18, 19, -26, -27], []]
    V = [[6, 7, 8, -15, -16, 23, 24, 25, -31, -32, -33], []]
    W = [[-4, -5, 12, 13, 14, -20, -21, -22, 28, 29, 30], []]
    kw1 = 0.954208
    kw5 = 0.187366
    kw7 = 0.131332
    ret = genwdg(Q, 2*p, m, None, layers) # 12/10
    wdglayout = ret['phases']
    print(wdglayout)
    assert [U, V, W] == wdglayout

    data = datamodel()
    data.set_machinedata(Q = Q, p = p, m = m)
    data.set_phases(wdglayout)
    data.analyse_wdg()
    #  pdb.set_trace()
    idx = data.results['nu_el'].index(1)
    assert kw1 == np.round(data.results['kw_el'][idx][0], 4) # phase U
    assert kw1 == np.round(data.results['kw_el'][idx][1], 4) # phase V
    assert kw1 == np.round(data.results['kw_el'][idx][2], 4) # phase W
    idx = data.results['nu_el'].index(5)
    assert kw5 == np.round(data.results['kw_el'][idx][0], 4) # phase U
    assert kw5 == np.round(data.results['kw_el'][idx][1], 4) # phase V
    assert kw5 == np.round(data.results['kw_el'][idx][2], 4) # phase W
    idx = data.results['nu_el'].index(7)
    assert kw7 == np.round(data.results['kw_el'][idx][0], 4) # phase U
    assert kw7 == np.round(data.results['kw_el'][idx][1], 4) # phase V
    assert kw7 == np.round(data.results['kw_el'][idx][2], 4) # phase W
"""










# -*- coding: utf-8 -*-
# Test for evaluating the radial force modes

import os
import sys

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))

from swat_em.wdggenerator import genwdg
from swat_em.datamodel import datamodel


def test_radial_force_modes1():
    data = datamodel()
    data.genwdg(Q=9, P=6, m=3, layers=2, w=1)
    print("Q:", data.get_num_slots(), "2p:", 2 * data.get_num_polepairs())
    modes = data.get_radial_force_modes(num_modes=4)
    print("radial force modes:", modes)
    assert modes == [3, 6, 9, 12]


def test_radial_force_modes2():
    data = datamodel()
    data.genwdg(Q=9, P=8, m=3, layers=2, w=1)
    print("Q:", data.get_num_slots(), "2p:", 2 * data.get_num_polepairs())
    modes = data.get_radial_force_modes(num_modes=4)
    print("radial force modes:", modes)
    assert modes == [1, 2, 3, 4]


def test_radial_force_modes3():
    data = datamodel()
    data.genwdg(Q=12, P=10, m=3, layers=2, w=1)
    print("Q:", data.get_num_slots(), "2p:", 2 * data.get_num_polepairs())
    modes = data.get_radial_force_modes(num_modes=4)
    print("radial force modes:", modes)
    assert modes == [2, 4, 6, 8]


def test_radial_force_modes4():
    data = datamodel()
    data.genwdg(Q=24, P=22, m=3, layers=2, w=1)
    print("Q:", data.get_num_slots(), "2p:", 2 * data.get_num_polepairs())
    modes = data.get_radial_force_modes(num_modes=4)
    print("radial force modes:", modes)
    assert modes == [2, 4, 6, 8]


if __name__ == "__main__":
    test_radial_force_modes1()
    test_radial_force_modes2()
    test_radial_force_modes3()
    test_radial_force_modes4()



# -*- coding: utf-8 -*-

from PyQt5 import uic
from PyQt5.QtWidgets import QDialog, QApplication, QMessageBox, QTableWidgetItem
from PyQt5.QtGui import QIntValidator
from PyQt5 import QtGui
from PyQt5 import QtCore
import numpy as np
import sys
import os

import pyqtgraph as pg
import pyqtgraph.exporters

pg.setConfigOption("background", "w")
pg.setConfigOption("foreground", "k")
pg.setConfigOptions(antialias=True)

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))
from swat_em import wdggenerator
from swat_em import datamodel
from swat_em.config import config, get_phase_color, get_line_color
from swat_em.report import num2str

if getattr(sys, "frozen", False) and hasattr(sys, "_MEIPASS"):
    __dir__ = sys._MEIPASS  # for pyinstaller
else:
    __dir__ = os.path.dirname(os.path.abspath(__file__))


class CombSniffer(QDialog):
    def __init__(self):
        super().__init__()

        # Set up the user interface from Designer.
        uic.loadUi(os.path.join(__dir__, "ui", "CombSniffer.ui"), self)

        self.plot_keys = [
            "idx",
            "Q",
            "P",
            "m",
            "q",
            "w",
            "sigma_d",
            "kw1",
            "a",
            "t",
            "r1",
            "lcmQP",
        ]

        for k in self.plot_keys:
            self.comboBox_xaxis.addItem(str(k))
            self.comboBox_yaxis.addItem(str(k))

        self.comboBox_xaxis.setCurrentIndex(6)  # sigma_d
        self.comboBox_yaxis.setCurrentIndex(7)  # kw1

        self.change_Qfix()
        self.change_Pfix()

        self.radioButton_Qfix.toggled.connect(self.change_Qfix)
        self.radioButton_Pfix.toggled.connect(self.change_Pfix)
        self.Button_find.clicked.connect(self.calc)
        self.Button_cp2clipboard_image.clicked.connect(self.cp2clipboard)
        self.comboBox_xaxis.currentIndexChanged.connect(self.update_plot)
        self.comboBox_yaxis.currentIndexChanged.connect(self.update_plot)
        self.checkBox_show_indices.toggled.connect(self.plot_indices)

        self.tableWidget.itemSelectionChanged.connect(self.on_table_selection)

        self.progressBar.setValue(0)

        self.init_plot()
        self.reset_data()

    def change_Qfix(self):
        if self.radioButton_Qfix.isChecked():
            self.spinBox_Q.setEnabled(True)
            self.spinBox_Q1.setEnabled(False)
            self.spinBox_Q2.setEnabled(False)
        else:
            self.spinBox_Q.setEnabled(False)
            self.spinBox_Q1.setEnabled(True)
            self.spinBox_Q2.setEnabled(True)

    def change_Pfix(self):
        if self.radioButton_Pfix.isChecked():
            self.spinBox_P.setEnabled(True)
            self.spinBox_P1.setEnabled(False)
            self.spinBox_P2.setEnabled(False)
        else:
            self.spinBox_P.setEnabled(False)
            self.spinBox_P1.setEnabled(True)
            self.spinBox_P2.setEnabled(True)

    def calc(self):

        # TODO: Clear all data in Comboboxes, plots and tables
        self.reset_data()

        if self.radioButton_Qfix.isChecked():
            Qrange = [self.spinBox_Q.value()]
        else:
            Qrange = list(range(self.spinBox_Q1.value(), self.spinBox_Q2.value() + 1))

        if self.radioButton_Pfix.isChecked():
            Prange = [self.spinBox_P.value()]
        else:
            Prange = list(
                range(self.spinBox_P1.value(), self.spinBox_P2.value() + 1, 2)
            )

        m = self.spinBox_m.value()

        layers = []
        if self.checkBox_singlelayer.isChecked():
            layers.append(1)
        if self.checkBox_doublelayer.isChecked():
            layers.append(2)
        if self.checkBox_emptyslots.isChecked():
            allow_empty_slots = True
        else:
            allow_empty_slots = False
        if allow_empty_slots:
            es = -1
        else:
            es = 0

        Qlist = []
        Plist = []
        wlist = []
        layerslist = []
        for Q_ in Qrange:
            for P_ in Prange:
                for l_ in layers:
                    if l_ == 2:
                        w = Q_ / P_
                        if w >= 2:
                            for w_ in range(1, int(w) + 1):
                                Qlist.append(Q_)
                                Plist.append(P_)
                                wlist.append(w_)
                                layerslist.append(l_)
                        else:
                            Qlist.append(Q_)
                            Plist.append(P_)
                            wlist.append(-1)
                            layerslist.append(l_)
                    else:
                        Qlist.append(Q_)
                        Plist.append(P_)
                        wlist.append(-1)
                        layerslist.append(l_)

        num_combinations = len(Qlist)
        #  print('Num combinations:', num_combinations)
        #  print('Qlist', Qlist)
        #  print('Plist', Plist)
        #  print('wlist', wlist)
        #  print('layerslist', layerslist)

        self.progressBar.setValue(0)
        self.wdg_list = []
        self.bc_list = []
        for k in range(num_combinations):
            wdg = datamodel()
            wdg.genwdg(
                Qlist[k],
                Plist[k],
                m=m,
                layers=layerslist[k],
                w=wlist[k],
                empty_slots=es,
                analyse=False,
            )
            kw1 = wdg.get_fundamental_windingfactor()
            #  print('Q', Qlist[k])
            #  print('P', Plist[k])
            #  print('w', wlist[k])
            #  print('layers', layerslist[k])
            #  print('kw1', kw1)

            if kw1 is not None:
                if kw1[0] > 0.01:
                    bc, bc_str = wdg.get_basic_characteristics()
                    if bc["sym"]:
                        self.wdg_list.append(wdg)
                        self.bc_list.append(bc)
            self.progressBar.setValue(100 / num_combinations * (k + 1))

        self.data = {}
        self.data["idx"] = list(range(len(self.wdg_list)))
        self.data["Q"] = [wdg.get_num_slots() for wdg in self.wdg_list]
        self.data["P"] = [2 * wdg.get_num_polepairs() for wdg in self.wdg_list]
        self.data["m"] = [wdg.get_num_phases() for wdg in self.wdg_list]
        self.data["q"] = [bc["q"] for bc in self.bc_list]
        self.data["w"] = []
        for wdg in self.wdg_list:
            w = wdg.get_windingstep()
            if type(w) == type([]):
                self.data["w"].append(np.mean(w))
            else:
                self.data["w"].append(w)

        self.data["kw1"] = [bc["kw1"][0] for bc in self.bc_list]
        self.data["sigma_d"] = [bc["sigma_d"] for bc in self.bc_list]
        self.data["a"] = [bc["a"] for bc in self.bc_list]
        self.data["t"] = [bc["t"] for bc in self.bc_list]
        self.data["r1"] = [bc["r"][0] for bc in self.bc_list]
        self.data["lcmQP"] = [bc["lcmQP"] for bc in self.bc_list]

        # make shure that keys for data are the same as the
        # items in the combo boxes of the axis to plot
        assert set(self.data.keys()) == set(self.plot_keys)

        self.update_data_in_gui()
        #  print('data', data)
        #  print('fertig')
        #  print('kw1:', kw1)

    def update_data_in_gui(self):
        self.update_plot()
        self.fill_table()
        self.textBrowser_wdginfo.setHtml(
            "{} valid windings found!<br><br>Click on a point in the \
        graphic window to get more informations!".format(
                len(self.wdg_list)
            )
        )

    def reset_data(self):
        """
        resetting all data before generating windings
        """
        self.data = {}
        self.fig.clear()
        self.indices_text = []
        self.scatter_marker_index = None
        self.textBrowser_wdginfo.setHtml('Click on "Find Winding"')
        self.tableWindingLayout.clear()
        self.tableWindingLayout.setRowCount(0)
        self.tableWindingLayout.setColumnCount(0)

        self.tableWidget.blockSignals(True)
        self.tableWidget.clear()
        self.tableWidget.setRowCount(0)
        self.tableWidget.setColumnCount(0)
        self.tableWidget.blockSignals(False)

    def init_plot(self):
        """
        generate figure in the gui
        """
        self.layout = self.mplvl
        self.widget = self.mplwidget
        self.fig = pg.PlotWidget()
        self.layout.addWidget(self.fig)
        self.fig.showGrid(x=True, y=True)

    def update_plot(self):
        """
        plot new data
        """
        self.fig.clear()
        # skip if there is no data to plot
        if len(self.data) == 0:
            return

        x_key = str(self.comboBox_xaxis.currentText())
        y_key = str(self.comboBox_yaxis.currentText())
        self.fig.setLabel("bottom", x_key)
        self.fig.setLabel("left", y_key)

        penlist = [pg.mkPen(None)] * len(self.data[x_key])
        size = [10] * len(self.data[x_key])

        self.scatter = pg.ScatterPlotItem()
        self.scatter.setData(self.data[x_key], self.data[y_key])
        self.scatter.setSize(size)
        self.scatter.setPen(penlist)
        self.scatter.sigClicked.connect(self.on_plot_click)
        self.fig.addItem(self.scatter)
        self.plot_indices()
        self.fig.autoRange()
        self.fig.show()

    def plot_marker(self, idx):
        """
        highlight point at index 'idx'
        """
        penlist = [pg.mkPen(None)] * len(self.scatter.getData()[0])
        size = [10] * len(penlist)
        if idx is not None:
            penlist[idx] = pg.mkPen(get_line_color(1), width=2)
            size[idx] = 15
        self.scatter.setPen(penlist)

    def plot_indices(self):
        """
        plot the index numbers of all points as strings
        """
        # skip if there is no data to plot
        if len(self.data) == 0:
            return

        if self.checkBox_show_indices.isChecked():
            x_key = str(self.comboBox_xaxis.currentText())
            y_key = str(self.comboBox_yaxis.currentText())
            for k in range(len(self.data[x_key])):
                txt = pg.TextItem(str(k), anchor=(0.0, 0.0))
                txt.setPos(self.data[x_key][k], self.data[y_key][k])
                txt.setColor("k")
                self.fig.addItem(txt, ignoreBounds=True)
                self.indices_text.append(txt)
        else:
            for txt in self.indices_text:
                self.fig.removeItem(txt)

    def on_plot_click(self, plot, points):
        """
        User clicked on the plot
        """
        idx = []
        for point in points:
            x, y = point.pos()
            idx1 = x == self.scatter.getData()[0]
            idx2 = y == self.scatter.getData()[1]
            i = np.logical_and(idx1, idx2)
            idx.append(np.where(i == True))
        idx = np.array(idx).flatten()
        idx = list(set(idx))

        # if there is one ore more points selected
        if len(idx) > 1:
            DIALOG = select_index(self.wdg_list, idx)
            idx = DIALOG.run()
            if idx is None:
                return
        else:
            idx = idx[0]
        self.combination_selected_in_table(idx)
        self.scatter_marker_index = idx
        self.plot_marker(idx)
        self.table_marker(idx)

    def cp2clipboard(self):
        """
        copy figure as bitmap to clipboard
        """
        img = self.widget.grab()
        QApplication.clipboard().setImage(img.toImage())

    def combination_selected_in_table(self, sel):
        """
        user selected a cell/winding combination
        """
        d = self.wdg_list[sel]
        bc, bc_text = d.get_basic_characteristics()
        self.textBrowser_wdginfo.setHtml(bc_text)

        table = self.tableWindingLayout
        table.clear()
        table.setRowCount(d.get_num_layers())
        table.setColumnCount(d.get_num_slots())

        l, ls, lcol = d.get_layers()
        for k1 in range(np.shape(l)[0]):
            for k2 in range(np.shape(l)[1]):
                table.setItem(k1, k2, QTableWidgetItem(ls[k1, k2]))
                item = table.item(k1, k2)
                if item:
                    item.setBackground(QtGui.QColor(lcol[k1, k2]))
                table.item(k1, k2).setFlags(
                    QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled
                )

        for k1 in range(d.get_num_slots()):
            table.resizeColumnToContents(k1)

    def fill_table(self):
        """
        fill table after generating winding combinations
        """
        # skip if there is no data to plot
        if len(self.data) == 0:
            return

        table = self.tableWidget
        table.clear()
        table.setRowCount(len(self.data["Q"]))
        table.setColumnCount(len(self.data))
        table.setSortingEnabled(True)

        myFont = QtGui.QFont()
        myFont.setBold(True)
        for i, key in enumerate(self.plot_keys):

            table_header_item = QTableWidgetItem(key)
            table_header_item.setFont(myFont)
            table.setHorizontalHeaderItem(i, table_header_item)

            for k in range(len(self.data[key])):
                txt = num2str(self.data[key][k], maxlen=8)
                table.setItem(k, i, QTableWidgetItem(txt))

                #  item = QTableWidgetItem()
                #  item.setData(QtCore.Qt.DisplayRole, self.data[key][k])
                #  self.table.setItem(k, i, item)

                table.item(k, i).setFlags(
                    QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled
                )  # note editable
            table.resizeColumnToContents(i)

    def on_table_selection(self):
        """
        user selected a winding in the table
        """
        #  row = mi.row()

        table = self.tableWidget
        row = table.currentRow()
        idx = int(table.item(row, 0).text())
        self.combination_selected_in_table(idx)
        self.scatter_marker_index = idx
        self.plot_marker(idx)

    def table_marker(self, idx):
        """
        Select winding in the table after selecting a point in the plot
        """
        table = self.tableWidget
        for row in range(table.rowCount()):
            i = int(table.item(row, 0).text())
            if i == idx:
                table.setCurrentCell(row, 0)
                break

    def run(self):
        ok = self.exec_()
        if ok:
            if self.scatter_marker_index is not None:

                if self.radioButton_overwrite_winding.isChecked():
                    overwrite = True
                else:
                    overwrite = False
                wdg = self.wdg_list[self.scatter_marker_index]
                w = self.data["w"][self.scatter_marker_index]
                if int(w) != w:
                    w = -1
                ret = {
                    "Q": wdg.get_num_slots(),
                    "P": 2 * wdg.get_num_polepairs(),
                    "m": wdg.get_num_phases(),
                    "w": w,
                    "layers": wdg.get_num_layers(),
                    "overwrite": overwrite,
                    "Qes": wdg.get_num_empty_slots(),
                }
                return ret
        return None


class select_index(QDialog):
    """
    Dialog for selecting an index of a winding while clicking on more
    than one point in the plot
    """

    def __init__(self, wdg_list, indices):
        super().__init__()
        uic.loadUi(os.path.join(__dir__, "ui", "CombSnifferSelIndix.ui"), self)
        self.wdg_list = wdg_list
        self.indices = indices
        self.listWidget.currentRowChanged.connect(self.index_changed)
        self.fill_table()

    def fill_table(self):
        """
        Show all selected indices
        """
        self.listWidget.blockSignals(True)
        for i in self.indices:
            self.listWidget.addItem(str(i))
        self.listWidget.blockSignals(False)
        self.listWidget.setCurrentRow(0)

    def index_changed(self):
        """
        User changed index
        """
        i = self.listWidget.currentRow()
        i = self.indices[i]
        bc, bc_text = self.wdg_list[i].get_basic_characteristics()
        self.textBrowser_wdginfo.setHtml(bc_text)

    def run(self):
        ok = self.exec_()
        if ok:
            i = self.listWidget.currentRow()
            return self.indices[i]
        else:
            return None



# -*- coding: utf-8 -*-

import os
import sys

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))

#  from swat.wdggenerator import genwdg, overlapping_fractional_slot_slayer, overlapping_fractional_slot_dlayer
from swat_em.wdggenerator import genwdg
from swat_em.datamodel import datamodel

#  import swat_em.analyse
import numpy as np
import pdb


def test1():
    print("Test double layer toothcoilwinding")
    Q = 18
    p = 8
    m = 3
    wstep = 1
    layers = 2
    #  U = [[1, -2, 3, 10, -11, 12], [-2, 3, -4, -11, 12, -13]]
    #  V = [[4, -5, 6, 13, -14, 15], [-5, 6, -7, -14, 15, -16]]
    #  W = [[7, -8, 9, 16, -17, 18], [-8, 9, -10, -17, 18, -1]]
    U = [[1, 8, -9, 10, 17, -18], [-2, -9, 10, -11, -18, 1]]
    V = [[2, -3, 4, 11, -12, 13], [-3, 4, -5, -12, 13, -14]]
    W = [[5, -6, 7, 14, -15, 16], [-6, 7, -8, -15, 16, -17]]
    kw1 = 0.9452
    kw5 = -0.1398
    kw7 = -0.0607
    ret = genwdg(Q, 2 * p, m, wstep, layers)
    wdglayout = ret["phases"]
    #  print(wdglayout)
    assert [U, V, W] == wdglayout

    data = datamodel()
    #  data.set_config(get_init_config())
    data.set_machinedata(Q=Q, p=p, m=m)
    data.set_phases(wdglayout)
    data.analyse_wdg()

    idx = data.results["nu_el"].index(1)
    assert kw1 == np.round(data.results["kw_el"][idx][0], 4)  # phase U
    assert kw1 == np.round(data.results["kw_el"][idx][1], 4)  # phase V
    assert kw1 == np.round(data.results["kw_el"][idx][2], 4)  # phase W
    idx = data.results["nu_el"].index(5)
    assert kw5 == np.round(data.results["kw_el"][idx][0], 4)  # phase U
    assert kw5 == np.round(data.results["kw_el"][idx][1], 4)  # phase V
    assert kw5 == np.round(data.results["kw_el"][idx][2], 4)  # phase W
    idx = data.results["nu_el"].index(7)
    assert kw7 == np.round(data.results["kw_el"][idx][0], 4)  # phase U
    assert kw7 == np.round(data.results["kw_el"][idx][1], 4)  # phase V
    assert kw7 == np.round(data.results["kw_el"][idx][2], 4)  # phase W

    bc, txt = data.get_basic_characteristics()
    #  print(bc)


def test2():
    print("Test single layer toothcoilwinding")
    Q = 18
    p = 8
    m = 3
    wstep = 1
    layers = 1
    #  U = [[1,-2,3,-4,-11,12]]
    #  V = [[-5,6,13,-14,15,-16]]
    #  W = [[7,-8,9,-10,-17,18]]
    U = [[1, -2, -9, 10, 17, -18], []]
    V = [[-3, 4, 11, -12, 13, -14], []]
    W = [[5, -6, 7, -8, -15, 16], []]

    kw1 = 0.9452
    kw5 = -0.1398
    kw7 = -0.0607
    ret = genwdg(Q, 2 * p, m, wstep, layers)  # 12/10
    wdglayout = ret["phases"]

    #  print(wdglayout)
    assert [U, V, W] == wdglayout

    data = datamodel()
    #  data.set_config(get_init_config())
    data.set_machinedata(Q=Q, p=p, m=m)
    data.set_phases(wdglayout)
    data.analyse_wdg()

    idx = data.results["nu_el"].index(1)
    assert kw1 == np.round(data.results["kw_el"][idx][0], 4)  # phase U
    assert kw1 == np.round(data.results["kw_el"][idx][1], 4)  # phase V
    assert kw1 == np.round(data.results["kw_el"][idx][2], 4)  # phase W
    idx = data.results["nu_el"].index(5)
    assert kw5 == np.round(data.results["kw_el"][idx][0], 4)  # phase U
    assert kw5 == np.round(data.results["kw_el"][idx][1], 4)  # phase V
    assert kw5 == np.round(data.results["kw_el"][idx][2], 4)  # phase W
    idx = data.results["nu_el"].index(7)
    assert kw7 == np.round(data.results["kw_el"][idx][0], 4)  # phase U
    assert kw7 == np.round(data.results["kw_el"][idx][1], 4)  # phase V
    assert kw7 == np.round(data.results["kw_el"][idx][2], 4)  # phase W


def test3():
    print("Test single layer toothcoilwinding")
    Q = 12
    p = 4
    m = 3
    wstep = 1
    layers = 1
    U = [[1, -2, 7, -8], []]
    V = [[5, -6, 11, -12], []]
    W = [[3, -4, 9, -10], []]
    kw1 = 0.866
    kw5 = -0.866
    kw7 = 0.866
    ret = genwdg(Q, 2 * p, m, wstep, layers)  # 12/10
    wdglayout = ret["phases"]
    #  print(wdglayout)
    assert [U, V, W] == wdglayout

    data = datamodel()
    data.set_machinedata(Q=Q, p=p, m=m)
    data.set_phases(wdglayout)
    data.analyse_wdg()

    idx = data.results["nu_el"].index(1)
    assert kw1 == np.round(data.results["kw_el"][idx][0], 4)  # phase U
    assert kw1 == np.round(data.results["kw_el"][idx][1], 4)  # phase V
    assert kw1 == np.round(data.results["kw_el"][idx][2], 4)  # phase W
    idx = data.results["nu_el"].index(5)
    assert kw5 == np.round(data.results["kw_el"][idx][0], 4)  # phase U
    assert kw5 == np.round(data.results["kw_el"][idx][1], 4)  # phase V
    assert kw5 == np.round(data.results["kw_el"][idx][2], 4)  # phase W
    idx = data.results["nu_el"].index(7)
    assert kw7 == np.round(data.results["kw_el"][idx][0], 4)  # phase U
    assert kw7 == np.round(data.results["kw_el"][idx][1], 4)  # phase V
    assert kw7 == np.round(data.results["kw_el"][idx][2], 4)  # phase W


def test4():
    print("Test single layer toothcoilwinding")
    Q = 12
    p = 5
    m = 3
    wstep = 1
    layers = 1
    U = [[1, -2, -7, 8], []]
    V = [[-3, 4, 9, -10], []]
    W = [[5, -6, -11, 12], []]
    kw1 = 0.9659
    kw5 = -0.2588
    kw7 = 0.2588
    ret = genwdg(Q, 2 * p, m, wstep, layers)  # 12/10
    wdglayout = ret["phases"]
    #  print(wdglayout)
    assert [U, V, W] == wdglayout

    data = datamodel()
    data.set_machinedata(Q=Q, p=p, m=m)
    data.set_phases(wdglayout)
    data.analyse_wdg()

    idx = data.results["nu_el"].index(1)
    assert kw1 == np.round(data.results["kw_el"][idx][0], 4)  # phase U
    assert kw1 == np.round(data.results["kw_el"][idx][1], 4)  # phase V
    assert kw1 == np.round(data.results["kw_el"][idx][2], 4)  # phase W
    idx = data.results["nu_el"].index(5)
    assert kw5 == np.round(data.results["kw_el"][idx][0], 4)  # phase U
    assert kw5 == np.round(data.results["kw_el"][idx][1], 4)  # phase V
    assert kw5 == np.round(data.results["kw_el"][idx][2], 4)  # phase W
    idx = data.results["nu_el"].index(7)
    assert kw7 == np.round(data.results["kw_el"][idx][0], 4)  # phase U
    assert kw7 == np.round(data.results["kw_el"][idx][1], 4)  # phase V
    assert kw7 == np.round(data.results["kw_el"][idx][2], 4)  # phase W


def test5():
    print("Test single layer toothcoilwinding")
    Q = 24
    p = 11
    m = 3
    wstep = 1
    layers = 1
    U = [[1, -2, -11, 12, -13, 14, 23, -24], []]
    V = [[-3, 4, -5, 6, 15, -16, 17, -18], []]
    W = [[7, -8, 9, -10, -19, 20, -21, 22], []]
    kw1 = 0.9577
    kw5 = -0.2053
    kw7 = -0.1576
    ret = genwdg(Q, 2 * p, m, wstep, layers)  # 12/10
    wdglayout = ret["phases"]
    #  print(wdglayout)
    assert [U, V, W] == wdglayout

    data = datamodel()
    data.set_machinedata(Q=Q, p=p, m=m)
    data.set_phases(wdglayout)
    data.analyse_wdg()

    idx = data.results["nu_el"].index(1)
    assert kw1 == np.round(data.results["kw_el"][idx][0], 4)  # phase U
    assert kw1 == np.round(data.results["kw_el"][idx][1], 4)  # phase V
    assert kw1 == np.round(data.results["kw_el"][idx][2], 4)  # phase W
    idx = data.results["nu_el"].index(5)
    assert kw5 == np.round(data.results["kw_el"][idx][0], 4)  # phase U
    assert kw5 == np.round(data.results["kw_el"][idx][1], 4)  # phase V
    assert kw5 == np.round(data.results["kw_el"][idx][2], 4)  # phase W
    idx = data.results["nu_el"].index(7)
    assert kw7 == np.round(data.results["kw_el"][idx][0], 4)  # phase U
    assert kw7 == np.round(data.results["kw_el"][idx][1], 4)  # phase V
    assert kw7 == np.round(data.results["kw_el"][idx][2], 4)  # phase W


def test6():
    print("Test single layer full pitch winding")
    Q = 24
    p = 2
    m = 3
    wstep = -1
    layers = 1
    U = [[1, 2, -7, -8, 13, 14, -19, -20], []]
    V = [[5, 6, -11, -12, 17, 18, -23, -24], []]
    W = [[-3, -4, 9, 10, -15, -16, 21, 22], []]
    kw1 = 0.9659
    kw5 = -0.2588
    kw7 = 0.2588
    #  wdglayout = genwdg(Q, 2*p, m, wstep, layers) # 12/10
    ret = genwdg(Q, 2 * p, m, wstep, layers)
    wdglayout = ret["phases"]
    #  print(wdglayout)
    #  pdb.set_trace()
    assert [U, V, W] == wdglayout

    data = datamodel()
    data.set_machinedata(Q=Q, p=p, m=m)
    data.set_phases(wdglayout)
    data.analyse_wdg()

    idx = data.results["nu_el"].index(1)
    assert kw1 == np.round(data.results["kw_el"][idx][0], 4)  # phase U
    assert kw1 == np.round(data.results["kw_el"][idx][1], 4)  # phase V
    assert kw1 == np.round(data.results["kw_el"][idx][2], 4)  # phase W
    idx = data.results["nu_el"].index(5)
    assert kw5 == np.round(data.results["kw_el"][idx][0], 4)  # phase U
    assert kw5 == np.round(data.results["kw_el"][idx][1], 4)  # phase V
    assert kw5 == np.round(data.results["kw_el"][idx][2], 4)  # phase W
    idx = data.results["nu_el"].index(7)
    assert kw7 == np.round(data.results["kw_el"][idx][0], 4)  # phase U
    assert kw7 == np.round(data.results["kw_el"][idx][1], 4)  # phase V
    assert kw7 == np.round(data.results["kw_el"][idx][2], 4)  # phase W


def test7():
    print("Test two layer full pitch winding")
    Q = 24
    p = 2
    m = 3
    wstep = 5
    layers = 2
    U = [[1, 2, -7, -8, 13, 14, -19, -20], [-6, -7, 12, 13, -18, -19, 24, 1]]
    V = [[5, 6, -11, -12, 17, 18, -23, -24], [-10, -11, 16, 17, -22, -23, 4, 5]]
    W = [[-3, -4, 9, 10, -15, -16, 21, 22], [8, 9, -14, -15, 20, 21, -2, -3]]
    kw1 = 0.9330
    kw5 = -0.0670
    kw7 = 0.0670
    ret = genwdg(Q, 2 * p, m, wstep, layers)  # 12/10
    wdglayout = ret["phases"]
    #  wdglayout = overlapping_fractional_slot_dlayer(Q, 2*p, m, wstep)
    #  print(wdglayout)
    #  pdb.set_trace()
    assert [U, V, W] == wdglayout

    data = datamodel()
    data.set_machinedata(Q=Q, p=p, m=m)
    data.set_phases(wdglayout)
    data.analyse_wdg()

    idx = data.results["nu_el"].index(1)
    assert kw1 == np.round(data.results["kw_el"][idx][0], 4)  # phase U
    assert kw1 == np.round(data.results["kw_el"][idx][1], 4)  # phase V
    assert kw1 == np.round(data.results["kw_el"][idx][2], 4)  # phase W
    idx = data.results["nu_el"].index(5)
    assert kw5 == np.round(data.results["kw_el"][idx][0], 4)  # phase U
    assert kw5 == np.round(data.results["kw_el"][idx][1], 4)  # phase V
    assert kw5 == np.round(data.results["kw_el"][idx][2], 4)  # phase W
    idx = data.results["nu_el"].index(7)
    assert kw7 == np.round(data.results["kw_el"][idx][0], 4)  # phase U
    assert kw7 == np.round(data.results["kw_el"][idx][1], 4)  # phase V
    assert kw7 == np.round(data.results["kw_el"][idx][2], 4)  # phase W


def test_8():
    print("test empty slots detection")
    Q = 12
    P = 8
    m = 3
    w = 1
    layers = 2

    wdg1 = datamodel()
    wdg1.genwdg(Q=Q, P=P, m=m, w=w, layers=layers, empty_slots=0)

    wdg2 = datamodel()
    wdg2.genwdg(
        Q=Q, P=P, m=m, w=w, layers=layers, empty_slots=-1
    )  # test for right detection
    assert wdg1.get_fundamental_windingfactor() == wdg2.get_fundamental_windingfactor()


def test_combinations_table():
    """
    Test a big table of different slot/pole combinations. The winding
    factors are taken from following paper:

    Libert, F. & Soulard, Juliette. (2004). Investigation on Pole-Slot Combinations for Permanent-Magnet Machines with Concentrated Windings. 

    https://www.researchgate.net/publication/237458473_Investigation_on_Pole-Slot_Combinations_for_Permanent-Magnet_Machines_with_Concentrated_Windings
    """
    print("Test a big table of different slot/pole combinations")

    wf = {}
    wf[6] = {
        4: 0.866,
        8: 0.866,
        10: 0.5,
        14: 0.5,
        16: 0.866,
        20: 0.866,
        22: 0.5,
        26: 0.5,
        28: 0.866,
        32: 0.866,
        34: 0.5,
        38: 0.5,
        40: 0.866,
    }
    wf[9] = {
        4: 0.616,
        6: 0.866,
        8: 0.945,
        10: 0.945,
        12: 0.866,
        14: 0.617,
        16: 0.328,
        20: 0.328,
        22: 0.617,
        24: 0.866,
        26: 0.945,
        28: 0.945,
        30: 0.866,
        32: 0.617,
        34: 0.328,
        38: 0.328,
        40: 0.617,
        42: 0.866,
    }
    wf[12] = {
        8: 0.866,
        10: 0.933,
        14: 0.933,
        16: 0.866,
        32: 0.866,
        34: 0.933,
        38: 0.933,
        40: 0.866,
    }
    wf[15] = {
        8: 0.621,
        10: 0.866,
        14: 0.951,
        16: 0.951,
        22: 0.621,
        38: 0.621,
        40: 0.866,
    }
    wf[18] = {
        10: 0.647,
        12: 0.866,
        14: 0.902,
        16: 0.945,
    }
    wf[21] = {14: 0.866, 16: 0.890, 20: 0.953, 22: 0.953, 26: 0.890, 28: 0.866}
    wf[24] = {
        14: 0.760,
        16: 0.866,
        20: 0.933,
        22: 0.950,
        26: 0.95,
        28: 0.933,
        32: 0.866,
        34: 0.760,
    }
    wf[27] = {
        18: 0.866,
        20: 0.877,
        22: 0.915,
        24: 0.945,
        26: 0.954,
        28: 0.954,
        30: 0.945,
        32: 0.915,
        34: 0.877,
        36: 0.866,
    }
    wf[30] = {
        20: 0.866,
        22: 0.874,
        26: 0.936,
        28: 0.951,
        32: 0.951,
        34: 0.936,
        38: 0.874,
        40: 0.866,
    }
    wf[33] = {
        22: 0.866,
        26: 0.903,
        28: 0.928,
        32: 0.954,
        34: 0.954,
        38: 0.928,
        40: 0.903,
    }
    wf[36] = {
        24: 0.866,
        26: 0.867,
        28: 0.902,
        30: 0.933,
        32: 0.945,
        34: 0.953,
        38: 0.953,
        40: 0.945,
        42: 0.933,
    }
    wf[39] = {26: 0.866, 28: 0.863, 32: 0.918, 34: 0.936, 38: 0.954, 40: 0.954}
    wf[42] = {28: 0.866, 32: 0.890, 34: 0.913, 38: 0.945, 40: 0.953}

    wf[45] = {
        42: 0.951,
        44: 0.955,
        46: 0.955,
        48: 0.951,
        50: 0.945,
        52: 0.927,
        56: 0.886,
        58: 0.859,
        60: 0.866,
    }
    wf[48] = {
        44: 0.950,
        46: 0.954,
        50: 0.954,
        52: 0.950,
        56: 0.933,
        58: 0.905,
        62: 0.857,
        64: 0.866,
    }
    wf[51] = {
        44: 0.933,
        46: 0.944,
        50: 0.955,
        52: 0.955,
        56: 0.944,
        58: 0.933,
        62: 0.901,
        64: 0.880,
        68: 0.866,
    }
    wf[54] = {
        42: 0.902,
        44: 0.915,
        46: 0.930,
        48: 0.945,
        50: 0.949,
        52: 0.954,
        56: 0.954,
        58: 0.949,
        60: 0.945,
        62: 0.930,
        64: 0.915,
        66: 0.902,
        68: 0.877,
        70: 0.854,
        72: 0.866,
    }
    wf[57] = {
        44: 0.932,
        46: 0.912,
        50: 0.937,
        52: 0.946,
        56: 0.955,
        58: 0.955,
        62: 0.946,
        64: 0.937,
        68: 0.912,
        70: 0.932,
        74: 0.852,
        76: 0.866,
    }
    wf[60] = {
        44: 0.874,
        46: 0.892,
        50: 0.933,
        52: 0.936,
        56: 0.951,
        58: 0.954,
        62: 0.954,
        64: 0.951,
        68: 0.936,
        70: 0.933,
        74: 0.892,
        76: 0.874,
        80: 0.866,
    }
    wf[63] = {
        42: 0.866,
        44: 0.850,
        46: 0.871,
        48: 0.890,
        50: 0.905,
        52: 0.919,
        56: 0.945,
        58: 0.948,
        60: 0.953,
        62: 0.955,
        64: 0.955,
        66: 0.953,
        68: 0.948,
        70: 0.945,
        74: 0.919,
        76: 0.905,
        78: 0.890,
        80: 0.871,
    }
    wf[66] = {
        44: 0.866,
        46: 0.849,
        50: 0.887,
        52: 0.903,
        56: 0.928,
        58: 0.938,
        62: 0.951,
        64: 0.954,
        68: 0.954,
        70: 0.951,
        74: 0.938,
        76: 0.928,
        80: 0.903,
    }
    wf[69] = {
        46: 0.866,
        50: 0.867,
        52: 0.884,
        56: 0.914,
        58: 0.925,
        62: 0.943,
        64: 0.949,
        68: 0.955,
        70: 0.955,
        74: 0.949,
        76: 0.943,
        80: 0.925,
    }
    wf[72] = {
        48: 0.866,
        50: 0.847,
        52: 0.867,
        56: 0.902,
        58: 0.911,
        60: 0.933,
        62: 0.933,
        64: 0.945,
        66: 0.950,
        68: 0.953,
        70: 0.954,
        74: 0.954,
        76: 0.953,
        78: 0.950,
        80: 0.945,
    }
    wf[75] = {
        50: 0.866,
        52: 0.846,
        56: 0.880,
        58: 0.895,
        62: 0.920,
        64: 0.930,
        68: 0.945,
        70: 0.951,
        74: 0.955,
        76: 0.955,
        80: 0.951,
    }
    wf[78] = {
        52: 0.866,
        56: 0.863,
        58: 0.879,
        62: 0.906,
        64: 0.918,
        68: 0.936,
        70: 0.943,
        74: 0.952,
        76: 0.954,
        80: 0.954,
    }
    wf[81] = {
        54: 0.866,
        56: 0.845,
        58: 0.860,
        60: 0.877,
        62: 0.890,
        64: 0.904,
        66: 0.915,
        68: 0.925,
        70: 0.933,
        72: 0.945,
        74: 0.946,
        76: 0.951,
        78: 0.954,
        80: 0.955,
    }
    wf[84] = {
        56: 0.866,
        58: 0.845,
        62: 0.876,
        64: 0.890,
        68: 0.913,
        70: 0.933,
        74: 0.939,
        76: 0.945,
        80: 0.953,
    }
    wf[87] = {
        58: 0.866,
        62: 0.859,
        64: 0.874,
        68: 0.899,
        70: 0.910,
        74: 0.929,
        76: 0.936,
        80: 0.947,
    }
    wf[90] = {
        60: 0.866,
        62: 0.843,
        64: 0.859,
        66: 0.874,
        68: 0.886,
        70: 0.902,
        74: 0.918,
        76: 0.927,
        78: 0.936,
        80: 0.945,
    }

    # Some of the windingfactors doesn't match;
    # these cases are excluded to prevent asserts
    exclude = [
        (15, 8),  # SWAT-EM finds a winding with a higher windingfactor
        (15, 22),  # SWAT-EM finds a winding with a higher windingfactor
        (15, 38),  # SWAT-EM finds a winding with a higher windingfactor
        (18, 10),  # SWAT-EM finds a winding with a higher windingfactor
        (57, 44),  # high windingfactor from the paper can't be reached here
        (57, 70),  # high windingfactor from the paper can't be reached here
        (81, 58),
    ]  # Wrong last digit in the paper??

    for slot, poles in wf.items():
        for P, wf in poles.items():
            if (slot, P) not in exclude:
                wdg = datamodel()
                wdg.genwdg(Q=slot, P=P, m=3, layers=2, w=1)
                wf2 = wdg.get_fundamental_windingfactor()[0]
                wf2 = np.round(np.abs(wf2), 3)
                if not np.allclose(wf, wf2, atol=0.001):
                    print(slot, P, wf, wf2)
                assert np.allclose(wf, wf2, atol=0.001)


def test_single_phase():
    print("Test single phase winding")
    wdg = datamodel()
    wdg.genwdg(Q=4, P=4, m=1, layers=2, w=1)
    assert wdg.get_fundamental_windingfactor()[0] == 1.0


"""
# Test is maybe not correct!!!
def test8():
    print('Test overlapping fractional slot winding with single layer')
    # TODO: Referenzdaten ergänzen
    Q = 33
    p = 2
    m = 3
    #  wstep = 1
    layers = 1
    U = [[1, 2, 3, -9, -10, -11, 17, 18, 19, -26, -27], []]
    V = [[6, 7, 8, -15, -16, 23, 24, 25, -31, -32, -33], []]
    W = [[-4, -5, 12, 13, 14, -20, -21, -22, 28, 29, 30], []]
    kw1 = 0.954208
    kw5 = 0.187366
    kw7 = 0.131332
    ret = genwdg(Q, 2*p, m, None, layers) # 12/10
    wdglayout = ret['phases']
    print(wdglayout)
    assert [U, V, W] == wdglayout

    data = datamodel()
    data.set_machinedata(Q = Q, p = p, m = m)
    data.set_phases(wdglayout)
    data.analyse_wdg()
    #  pdb.set_trace()
    idx = data.results['nu_el'].index(1)
    assert kw1 == np.round(data.results['kw_el'][idx][0], 4) # phase U
    assert kw1 == np.round(data.results['kw_el'][idx][1], 4) # phase V
    assert kw1 == np.round(data.results['kw_el'][idx][2], 4) # phase W
    idx = data.results['nu_el'].index(5)
    assert kw5 == np.round(data.results['kw_el'][idx][0], 4) # phase U
    assert kw5 == np.round(data.results['kw_el'][idx][1], 4) # phase V
    assert kw5 == np.round(data.results['kw_el'][idx][2], 4) # phase W
    idx = data.results['nu_el'].index(7)
    assert kw7 == np.round(data.results['kw_el'][idx][0], 4) # phase U
    assert kw7 == np.round(data.results['kw_el'][idx][1], 4) # phase V
    assert kw7 == np.round(data.results['kw_el'][idx][2], 4) # phase W
"""


if __name__ == "__main__":
    test1()
    test2()
    test3()
    test4()
    test5()
    test6()
    test7()
    test_8()
    test_combinations_table()
    test_single_phase()



from PyQt5 import uic, QtGui
from PyQt5.QtWidgets import QDialog
import os
import sys

if getattr(sys, "frozen", False) and hasattr(sys, "_MEIPASS"):
    __dir__ = sys._MEIPASS  # for pyinstaller
else:
    __dir__ = os.path.dirname(os.path.abspath(__file__))


class get_notes(QDialog):
    def __init__(self, init_text=""):
        self.init_text = init_text
        super().__init__()
        uic.loadUi(os.path.join(__dir__, "ui", "Notes.ui"), self)

    def run(self):
        self.plainTextEdit.setPlainText(self.init_text)
        self.plainTextEdit.moveCursor(QtGui.QTextCursor.End)

        ok = self.exec_()
        if ok:
            return str(self.plainTextEdit.toPlainText())
        else:
            return None



# -*- coding: utf-8 -*-
# Test for evaluating the radial force modes

import os
import sys

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))

from swat_em.wdggenerator import genwdg
from swat_em.datamodel import datamodel


def test_num_basic_winding():
    # 2-layer
    data = datamodel()
    data.genwdg(Q=12, P=8, m=3, layers=2, w=1)
    bc, _ = data.get_basic_characteristics()
    t = data.calc_num_basic_windings_t()
    assert bc["t"] == t

    data = datamodel()
    data.genwdg(Q=12, P=10, m=3, layers=2, w=1)
    bc, _ = data.get_basic_characteristics()
    t = data.calc_num_basic_windings_t()
    assert bc["t"] == t

    data.genwdg(Q=18, P=6, m=3, layers=2, w=-1)
    bc, _ = data.get_basic_characteristics()
    t = data.calc_num_basic_windings_t()
    assert bc["t"] == t

    data.genwdg(Q=18, P=12, m=3, layers=2, w=-1)
    bc, _ = data.get_basic_characteristics()
    t = data.calc_num_basic_windings_t()
    assert bc["t"] == t

    data.genwdg(Q=30, P=4, m=3, layers=2, w=1)
    bc, _ = data.get_basic_characteristics()
    t = data.calc_num_basic_windings_t()
    assert bc["t"] == t

    # 1-layer
    data = datamodel()
    data.genwdg(Q=12, P=10, m=3, layers=1, w=1)
    bc, _ = data.get_basic_characteristics()
    t = data.calc_num_basic_windings_t()
    assert bc["t"] == t

    # this winding-layout may not be correct
    #  data = datamodel()
    #  data.genwdg(Q = 30, P = 8, m = 3, layers = 1, w = -1)
    #  bc, _ = data.get_basic_characteristics()
    #  t = data.calc_num_basic_windings_t()
    #  assert bc['t'] == t


if __name__ == "__main__":
    test_num_basic_winding()



# -*- coding: utf-8 -*-

import os
import sys

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))


from swat_em.wdggenerator import genwdg

#  from swat_em.datamodel import datamodel
from swat_em import datamodel
from swat_em.config import config, get_phase_color


def clean_up():
    try:
        os.remove("export.xlsx")
    except:
        pass


def test_xlsx_export():
    clean_up()
    data = datamodel()
    data.set_title("Q = 12, 2p = 10")
    data.set_notes("Example winding")
    data.genwdg(Q=12, P=10, m=3, w=1, layers=2)
    data.analyse_wdg()
    bc, txt = data.get_basic_characteristics()
    data.export_xlsx("export.xlsx")
    assert os.path.isfile("export.xlsx")
    clean_up()


if __name__ == "__main__":
    test_xlsx_export()


#  import os
#  os.system('loffice export.xlsx')



from PyQt5 import uic
from PyQt5 import QtGui
from PyQt5.QtWidgets import QDialog, QTableWidgetItem, QMessageBox
from PyQt5.QtGui import QDoubleValidator
import numpy as np
from swat_em.config import get_phase_color, config
from swat_em.analyse import _get_float
import os
import sys

if getattr(sys, "frozen", False) and hasattr(sys, "_MEIPASS"):
    __dir__ = sys._MEIPASS  # for pyinstaller
else:
    __dir__ = os.path.dirname(os.path.abspath(__file__))


def get_int_from_str(txt):
    """
    converts txt to a integer number and returns this number
    return None if txt is not a number
    """
    try:
        number = int(txt)
        if number != 0:
            return number
    except:
        return None


class layout(QDialog):
    """
    This is a basic winding editor to define and change the 
    winding layout
    """

    def __init__(self, data):
        self.data = data
        super().__init__()
        # Set up the user interface from Designer.
        uic.loadUi(os.path.join(__dir__, "ui", "winding_layout.ui"), self)
        self.setWindowTitle("Edit winding layout")

        self.Button_EditMachineData.clicked.connect(self.dialog_machinedata)
        #  self.buttonBox.accepted.connect(self.accept)

        self.lineEditFixTurns.setValidator(QDoubleValidator())

        self.radioTurnsFix.toggled.connect(self.update_radio_turns)
        self.tableWindingLayout.cellChanged.connect(self.update_colors)
        self.tableWindingTurns.cellChanged.connect(self.update_colors)
        self.update_table()
        self.update_table_turns()
        if type(self.data.get_turns()) == type([]):
            self.radioTurnsIndividual.setChecked(True)
        else:
            self.radioTurnsFix.setChecked(True)
            self.lineEditFixTurns.setText(str(self.data.get_turns()))

        self.update_lineEdits(
            self.data.get_num_slots(),
            self.data.get_num_phases(),
            2 * self.data.get_num_polepairs(),
            self.data.get_num_layers(),
        )

    def dialog_machinedata(self):
        """
        User can change Q, m, P, layers
        """
        dialog = machinedata(self.data)
        ret = dialog.run()
        make_update = False
        if ret:
            if ret["Q"] != self.data.get_num_slots():
                make_update = True
            if ret["layers"] != self.data.get_num_layers():
                make_update = True
                for km in range(len(self.data.get_phases())):
                    for ks in range(1, len(self.data.get_phases()[km])):
                        self.data.get_phases()[km][ks] = []
            self.data.set_machinedata(Q=ret["Q"], m=ret["m"], p=ret["P"] / 2)
            self.update_lineEdits(
                self.data.get_num_slots(),
                self.data.get_num_phases(),
                2 * self.data.get_num_polepairs(),
                ret["layers"],
            )
            if make_update:
                self.data.set_phases(self.read_layout())
                self.update_table(layers=ret["layers"])
                self.update_table_turns()

    def update_radio_turns(self):
        if self.radioTurnsFix.isChecked():
            self.tableWindingTurns.setEnabled(False)
            self.lineEditFixTurns.setEnabled(True)
        else:
            self.tableWindingTurns.setEnabled(True)
            self.lineEditFixTurns.setEnabled(False)
            self.update_colors()

    def update_lineEdits(self, Q, m, P, layers):
        self.lineEdit_Q.setText(str(Q))
        self.lineEdit_m.setText(str(m))
        self.lineEdit_P.setText(str(P))
        if layers == 1:
            self.radioButton_slayer.setChecked(True)
        else:
            self.radioButton_dlayer.setChecked(True)

    def update_table(self, layers=None):
        self.table = self.tableWindingLayout
        self.table.blockSignals(True)
        self.table.setColumnCount(self.data.get_num_slots())

        head = ["Layer 1", "Layer 2"]
        if not layers:
            layers = self.data.get_num_layers()
        self.table.setRowCount(layers)
        self.table.setVerticalHeaderLabels(head[:layers])

        l, ls, lcol = self.data.get_layers()
        for k1 in range(np.shape(l)[0]):
            for k2 in range(np.shape(l)[1]):
                self.table.setItem(k1, k2, QTableWidgetItem(ls[k1, k2]))
                item = self.table.item(k1, k2)
                if item:
                    item.setBackground(QtGui.QColor(lcol[k1, k2]))

        for k1 in range(self.data.get_num_slots()):
            self.table.resizeColumnToContents(k1)
        self.table.blockSignals(False)

    def update_table_turns(self):
        table = self.tableWindingTurns
        Nx = self.tableWindingLayout.columnCount()
        Ny = self.tableWindingLayout.rowCount()
        table.setRowCount(Ny)
        table.setColumnCount(Nx)

        head = ["Layer 1", "Layer 2"]
        layers = self.data.get_num_layers()
        table.setRowCount(layers)
        table.setVerticalHeaderLabels(head[:layers])

        phases = self.data.get_phases()
        turns = self.data.get_turns()

        for km, ph in enumerate(phases):
            col = get_phase_color(km)
            for kl in range(len(ph)):
                layer = ph[kl]
                for i, cs in enumerate(layer):
                    if type(turns) == type([]):
                        item = QTableWidgetItem(str(turns[km][kl][i]))
                    else:
                        item = QTableWidgetItem(str(turns))
                    item.setBackground(QtGui.QColor(col))
                    if cs > 0:
                        table.setItem(kl, cs - 1, item)
                    else:
                        table.setItem(kl, abs(cs) - 1, item)

        # test if there are cells without a value (occures if Q is changed)
        for k1 in range(self.data.get_num_layers()):
            for k2 in range(self.data.get_num_slots()):
                item = table.item(k1, k2)
                if item == None:
                    item = QTableWidgetItem("0")
                    table.setItem(k1, k2, item)

        for k1 in range(self.data.get_num_slots()):
            table.resizeColumnToContents(k1)

    def read_layout(self):
        """
        Read layout from table
        """
        S = []
        for layer in [0, 1]:
            for k in range(self.data.get_num_slots()):
                item = self.table.item(layer, k)
                if item:
                    txt = str(item.text())
                    num = get_int_from_str(txt)
                    if num:
                        phase = num
                        sign = 1 if phase > 0 else -1
                        while len(S) < abs(phase):
                            S.append([[], []])
                        S[abs(phase) - 1][layer].append((k + 1) * sign)
        return S

    def read_turns(self):
        """
        Read layout from table
        """
        table = self.tableWindingTurns
        if self.radioTurnsFix.isChecked():
            return None
        else:
            S = []
            for layer in range(table.rowCount()):
                for k in range(self.data.get_num_slots()):
                    item = self.tableWindingLayout.item(layer, k)  # phase
                    item2 = table.item(layer, k)  # turns
                    if item:
                        txt = str(item.text())
                        num = get_int_from_str(txt)
                        if num:
                            phase = num
                            sign = 1 if phase > 0 else -1
                            while len(S) < abs(phase):
                                S.append([[], []])
                            fl = _get_float(item2.text())
                            if fl is not None:
                                S[abs(phase) - 1][layer].append(fl)
                            else:
                                S[abs(phase) - 1][layer].append(0.0)
        return S

    def update_colors(self):
        """
        Update the colors in the table if the user defines phases here
        Also there are some tests for the winding
        """
        error = []
        warning = []

        for layer in [0, 1]:
            for k in range(self.data.get_num_slots()):
                item = self.table.item(layer, k)
                item2 = self.tableWindingTurns.item(layer, k)
                if item:
                    txt = str(item.text())
                    num = get_int_from_str(txt)
                    if num:
                        phase = abs(num)
                        col = get_phase_color(phase - 1)
                        item.setBackground(QtGui.QColor(col))
                        if not self.radioTurnsFix.isChecked():
                            item2.setBackground(QtGui.QColor(col))
                            fl = _get_float(item2.text())
                            if fl is None:
                                item2.setText("0")
                    else:
                        item.setBackground(QtGui.QColor("white"))
                        if not self.radioTurnsFix.isChecked():
                            item2.setBackground(QtGui.QColor("white"))
                            fl = _get_float(item2.text())
                            if fl is None:
                                item2.setText("0")

        # Test for errors
        S = self.read_layout()
        T = self.read_turns()
        for k in range(len(S)):
            S[k] = [item for sublist in S[k] for item in sublist]  # flatten layers
            if T is not None:
                T[k] = [item for sublist in T[k] for item in sublist]  # flatten layers

        # test if the number of positve and negative coil sides are equal
        for k in range(len(S)):
            s = S[k]
            pos = 0
            neg = 0
            for w in s:
                if w > 0:
                    pos += 1
                elif w < 0:
                    neg += 1
            if pos != neg:
                error.append(
                    "Phase {} has {} postive and {} negative coil sides".format(
                        k + 1, pos, neg
                    )
                )

        # Test if theta is 0 for all phases
        if T is not None:
            theta = []
            for km in range(len(S)):
                theta.append(0.0)
                for kp in range(len(S[km])):
                    sign = 1 if S[km][kp] > 0 else -1
                    theta[km] += sign * T[km][kp]
            for i, k in enumerate(theta):
                if abs(k) > 1e-3:
                    txt = "Theta (SUM(Turns*SignOfCoilSides)) of Phase {} is not zero. Check number of turns if there is no error listed regarding the coil sides:<br>".format(
                        i + 1
                    )
                    error.append(txt)

        # test if all phases have the same number of coil sides
        l = [len(s) for s in S]
        if len(set(l)) != 1:
            txt = "Not all phases have the same number of coil sides:<br>"
            for k in range(len(S)):
                txt += "Phase {} hat {} coilsides<br>".format(k + 1, l[k])
            error.append(txt)

        if len(S) > self.data.get_num_phases():
            error.append(
                "There are {} phases but in main window only m = {} is set".format(
                    len(S), self.data.get_num_phases()
                )
            )

        # print errors
        txt_error = '<span style=" color:#ff0000;" >'
        for e in error:
            txt_error += str(e) + "<br>"
        txt_error += "</span>"
        self.textBrowser_output.setHtml(txt_error)

    def accept(self):
        """overwrite the dialog signal for exiting the exec_ command;
        only exit when all user inputs are correct and validated"""
        error = None
        if self.radioTurnsFix.isChecked():
            txt = self.lineEditFixTurns.text()
            turns = _get_float(txt)
            if turns == None:
                QMessageBox.critical(
                    self, "Error", '"{}" is not a valid number of turns'.format(txt)
                )
                error = True
            if type(self.data.get_turns()) == type([]):
                ret = QMessageBox.question(
                    self,
                    "Overwrite number of turns",
                    "In the previous winding there are individual number of turns. Is it ok to loose them?",
                    QMessageBox.Yes | QMessageBox.Cancel,
                )
                if ret != QMessageBox.Yes:
                    error = False

        if error is None:
            super().accept()  # call accept Dialog from Qdialog (end of exec_)

    def run(self):
        ok = self.exec_()
        if ok:
            if self.radioButton_overwrite_winding.isChecked():
                overwrite = True
            else:
                overwrite = False
            phases = self.read_layout()

            if self.radioTurnsFix.isChecked():
                txt = self.lineEditFixTurns.text()
                turns = _get_float(txt)
            else:
                turns = self.read_turns()

            layer1 = False
            layer2 = False  # test if single or double layer winding
            for s in phases:
                if len(s[0]) > 0:
                    layer1 = True
                if len(s[1]) > 0:
                    layer2 = True
            if layer1 and layer2:
                layers = 2
            else:
                layers = 1

            Q = self.data.get_num_slots()
            p = self.data.get_num_polepairs()
            m = self.data.get_num_phases()
            w = self.data.get_windingstep()
            ret = {
                "phases": phases,
                "Q": Q,
                "P": 2 * p,
                "m": m,
                "w": w,
                "layers": layers,
                "overwrite": overwrite,
                "turns": turns,
            }
            return ret
        else:
            return None


class machinedata(QDialog):
    """
    User can change Q, m, P, layers
    """

    def __init__(self, data):
        self.data = data
        super().__init__()
        uic.loadUi(os.path.join(__dir__, "ui", "MachineData.ui"), self)
        self.setWindowTitle("Change machine data")
        self.data = data
        self.spinBox_Q.setValue(self.data.get_num_slots())
        self.spinBox_m.setValue(self.data.get_num_phases())
        self.spinBox_P.setValue(2 * self.data.get_num_polepairs())
        if self.data.get_num_layers() == 1:
            self.radioButton_slayer.setChecked(True)
        else:
            self.radioButton_dlayer.setChecked(True)

    def run(self):
        ok = self.exec_()
        if ok:
            if self.radioButton_slayer.isChecked():
                layers = 1
            elif self.radioButton_dlayer.isChecked():
                layers = 2
            Q = self.spinBox_Q.value()
            P = self.spinBox_P.value()
            m = self.spinBox_m.value()
            if Q < self.data.get_num_slots():
                buttonReply = QMessageBox.question(
                    self,
                    "Warning",
                    """The defined number of slots is lower than the number of slots in the actual winding. \
Is it ok to loose some informations of the actual winding layout?""",
                    QMessageBox.Yes | QMessageBox.No,
                    QMessageBox.No,
                )
                if buttonReply != QMessageBox.Yes:
                    return None

            if layers < self.data.get_num_layers():
                buttonReply = QMessageBox.question(
                    self,
                    "Warning",
                    """The defined number of layers is lower than the number of layers in the actual winding. \
Is it ok to loose some informations of the actual winding layout?""",
                    QMessageBox.Yes | QMessageBox.No,
                    QMessageBox.No,
                )
                if buttonReply != QMessageBox.Yes:
                    return None

            ret = {"Q": Q, "P": P, "m": m, "layers": layers}
            return ret
        else:
            return None



# -*- coding: utf-8 -*-

import os
import sys

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))

from swat_em.datamodel import datamodel


def test_image_export():
    def clean_up():
        for f in ["plot_layout.png", "plot_star.png", "plot_wf.png", "plot_MMK.png"]:
            try:
                os.remove(f)
            except:
                pass

    data = datamodel()
    data.genwdg(Q=12, P=2, m=3, w=-1, layers=1, turns=1000)
    data.analyse_wdg()

    print("Test image export")
    clean_up()  # clear old data

    data.plot_layout("plot_layout.png")
    assert os.path.isfile("plot_layout.png")

    data.plot_star("plot_star.png")
    assert os.path.isfile("plot_star.png")

    data.plot_windingfactor("plot_wf.png", mechanical=False)
    assert os.path.isfile("plot_wf.png")

    data.plot_MMK("plot_MMK.png", res=[800, 600], phase=0)
    assert os.path.isfile("plot_MMK.png")

    clean_up()  # clear data


if __name__ == "__main__":
    test_image_export()

    # interactive usage:
    #  data = datamodel()
    #  data.genwdg(Q = 12, P = 2, m = 3, w = -1, layers = 1, turns = 1000)
    #  data.analyse_wdg()
    #  data.plot_layout('plot_layout.png', res = [1200, 900], show = True)



# -*- coding: utf-8 -*-

from PyQt5 import uic
from PyQt5.QtWidgets import QDialog, QApplication, QMessageBox, QTableWidgetItem
from PyQt5.QtGui import QIntValidator
from PyQt5 import QtGui
from PyQt5 import QtCore

#  from PyQt5.QtCore import Qt
import numpy as np
import sys
import os

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))
from swat_em import wdggenerator
from swat_em import datamodel
from swat_em.config import config, get_phase_color

if getattr(sys, "frozen", False) and hasattr(sys, "_MEIPASS"):
    __dir__ = sys._MEIPASS  # for pyinstaller
else:
    __dir__ = os.path.dirname(os.path.abspath(__file__))


class NewWinding(QDialog):
    def __init__(self):
        super().__init__()
        uic.loadUi(os.path.join(__dir__, "ui", "choose_generator.ui"), self)
        self.generator = None

        self.Button_EditWindingLayout.clicked.connect(lambda: self.ret(generator=0))
        self.Button_GenerateAutomatically.clicked.connect(lambda: self.ret(generator=1))
        self.Button_FindByTable.clicked.connect(lambda: self.ret(generator=2))

    def run(self):
        ok = self.exec_()
        if ok:
            return self.generator
        else:
            return None

    def ret(self, generator):
        """saves the choosen generator and say "ok" to the dialog"""
        self.generator = generator
        self.accept()


class GenWinding2(QDialog):
    def __init__(self):
        super().__init__()

        # Set up the user interface from Designer.
        uic.loadUi(os.path.join(__dir__, "ui", "GenWinding2.ui"), self)
        self.setWindowTitle("Generate winding")

        self.radioButton_slayer.toggled.connect(
            self.change_layers
        )  # disable lineEdit for single phase windings

        # update 'w' and update winding
        self.spinBox_Q.valueChanged.connect(self.QmP_changed)
        self.spinBox_P.valueChanged.connect(self.QmP_changed)
        self.spinBox_m.valueChanged.connect(self.QmP_changed)
        self.spinBox_empty_slots.valueChanged.connect(self.QmP_changed)

        # update winding
        self.radioButton_slayer.toggled.connect(self.update)
        self.spinBox_w.valueChanged.connect(self.update)

        self.update()  # calc initial winding

    def change_layers(self):
        """
        disable lineEdit for single phase windings
        """
        if self.radioButton_slayer.isChecked():
            self.spinBox_w.setEnabled(False)
        else:
            self.spinBox_w.setEnabled(True)

    def QmP_changed(self):
        Q = self.spinBox_Q.value()
        P = self.spinBox_P.value()
        m = self.spinBox_m.value()
        w = Q // P  # take smaller value (integer div.)
        if w <= 0:
            w = 1
        self.spinBox_w.setValue(w)
        self.update()

    def update(self):
        self.Q = self.spinBox_Q.value()
        self.P = self.spinBox_P.value()
        self.m = self.spinBox_m.value()
        self.w = self.spinBox_w.value()
        Qes = self.spinBox_empty_slots.value()

        if self.radioButton_slayer.isChecked():
            self.layers = 1
        elif self.radioButton_dlayer.isChecked():
            self.layers = 2

        self.table = self.tableWindingLayout
        self.table.clear()
        self.textBrowser_wdginfo.clear()

        self.data = datamodel()
        self.data.set_machinedata(Q=self.Q, m=self.m, p=self.P / 2)
        ret = wdggenerator.genwdg(self.Q, self.P, self.m, self.w, self.layers, Qes)
        if ret is None:
            return
        if ret["valid"] == False:
            return

        self.data.set_phases(S=ret["phases"], wstep=ret["wstep"])
        self.data.set_valid(ret["valid"], ret["error"], ret["info"])
        self.data.set_num_empty_slots(ret["Qes"])
        bc, bc_str = self.data.get_basic_characteristics()
        self.textBrowser_wdginfo.setHtml(bc_str)

        if bc["sym"] and self.data.generator_info["valid"]:
            self.table.setRowCount(self.layers)
            self.table.setColumnCount(self.data.get_num_slots())

            l, ls, lcol = self.data.get_layers()
            for k1 in range(np.shape(l)[0]):
                for k2 in range(np.shape(l)[1]):
                    self.table.setItem(k1, k2, QTableWidgetItem(ls[k1, k2]))
                    item = self.table.item(k1, k2)
                    if item:
                        item.setBackground(QtGui.QColor(lcol[k1, k2]))
                    self.table.item(k1, k2).setFlags(
                        QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled
                    )

            for k1 in range(self.data.get_num_slots()):
                #  self.table.resizeColumnToContents(k1)
                self.table.setColumnWidth(k1, 25)

    def run(self):
        ok = self.exec_()
        if ok:
            if self.radioButton_overwrite_winding.isChecked():
                overwrite = True
            else:
                overwrite = False
            ret = {
                "Q": self.Q,
                "P": self.P,
                "m": self.m,
                "w": self.w,
                "layers": self.layers,
                "overwrite": overwrite,
                "Qes": self.data.get_num_empty_slots(),
            }
            return ret
        else:
            return None


class GenWindingCombinations(QDialog):
    def __init__(self):
        super().__init__()

        uic.loadUi(os.path.join(__dir__, "ui", "GenCombinations.ui"), self)
        self.setWindowTitle("Generate windings per table")

        self.colorlabel1.setStyleSheet("background-color:#ADD8E6;")
        self.colorlabel2.setStyleSheet("background-color:#E6C3AD;")
        self.colorlabel3.setStyleSheet("background-color:#BCFFBC;")
        self.colorlabel4.setStyleSheet("background-color:#E9AEE2;")

        # update 'w' and update winding
        self.spinBox_Q1.valueChanged.connect(self.generate)
        self.spinBox_Q2.valueChanged.connect(self.generate)
        self.spinBox_P1.valueChanged.connect(self.generate)
        self.spinBox_P2.valueChanged.connect(self.generate)
        self.spinBox_m.valueChanged.connect(self.generate)
        self.radioButton_slayer.toggled.connect(self.generate)
        self.checkBox_toothcoil.toggled.connect(self.generate)
        self.checkBox_empty_slots.toggled.connect(self.generate)

        self.tableCombinations.itemSelectionChanged.connect(self.combination_selected)

        # Fill combobox
        self.comboBox_plotval.addItems(
            [
                "kw1 (winding factor)",
                "q (slots per pole per phase)",
                "t (symmetry)",
                "a (parallel circuit)",
                "lcm(Q,2p) (least common multiple)",
                "r1 (first mode of radial force)",
                "sigma_d (double linked leakage)",
            ]
        )
        self.comboBox_plotval.currentIndexChanged.connect(self.update_table)
        #  self.generate()

    def generate(self):
        """
        Generate possible winding layout for different slot/pole combinations
        """
        self.Q1 = self.spinBox_Q1.value()
        self.Q2 = self.spinBox_Q2.value()
        self.P1 = self.spinBox_P1.value()
        self.P2 = self.spinBox_P2.value()
        self.m = self.spinBox_m.value()
        if self.checkBox_toothcoil.isChecked():
            wstep = 1
        else:
            wstep = -1
        if self.checkBox_empty_slots.isChecked():
            empty_slots = -1
        else:
            empty_slots = 0

        if self.radioButton_slayer.isChecked():
            self.layers = 1
        elif self.radioButton_dlayer.isChecked():
            self.layers = 2

        self.Qlist = []
        for k in range(self.Q1, self.Q2 + 1, 1):
            if k % self.m == 0:
                self.Qlist.append(k)
        #  self.Qlist = list(range(self.Q1, self.Q2+1, self.m))
        self.Plist = list(range(self.P1, self.P2 + 1, 2))

        self.data = []
        for iQ, kQ in enumerate(self.Qlist):
            self.data.append([])
            for iP, kP in enumerate(self.Plist):
                d = datamodel()
                d.set_machinedata(Q=kQ, m=self.m, p=kP / 2)
                ret = wdggenerator.genwdg(
                    kQ, kP, self.m, wstep, self.layers, empty_slots
                )
                if ret:
                    d.set_phases(S=ret["phases"], wstep=ret["wstep"])
                    d.set_valid(ret["valid"], ret["error"], ret["info"])
                    d.set_num_empty_slots(ret["Qes"])
                    bc, bc_str = d.get_basic_characteristics()
                else:
                    d.set_valid(False, "", "")
                self.data[iQ].append(d)
        self.update_table()

    def update_table(self):
        self.table = self.tableCombinations
        self.table.clear()

        self.table.setRowCount(len(self.Qlist))
        self.table.setColumnCount(len(self.Plist))

        for i in range(len(self.Plist)):
            table_header_item = QTableWidgetItem("2p=" + str(self.Plist[i]))
            myFont = QtGui.QFont()
            myFont.setBold(True)
            table_header_item.setFont(myFont)
            self.table.setHorizontalHeaderItem(i, table_header_item)

        for i in range(len(self.Qlist)):
            table_header_item = QTableWidgetItem("Q=" + str(self.Qlist[i]))
            myFont = QtGui.QFont()
            myFont.setBold(True)
            table_header_item.setFont(myFont)
            self.table.setVerticalHeaderItem(i, table_header_item)

        for iQ, kQ in enumerate(self.Qlist):
            self.data.append([])
            for iP, kP in enumerate(self.Plist):
                if (
                    self.data[iQ][iP].generator_info["valid"]
                    and self.data[iQ][iP].results["basic_char"]["sym"]
                ):
                    bc = self.data[iQ][iP].results["basic_char"]
                    if bc["kw1"][0] > 0.01:
                        idx = self.comboBox_plotval.currentIndex()
                        if idx == 0:
                            txt = str(round(bc["kw1"][0], 3))
                        elif idx == 1:
                            txt = str(bc["q"])
                        elif idx == 2:
                            txt = str(bc["t"])
                        elif idx == 3:
                            txt = str(bc["a"])
                        elif idx == 4:
                            txt = str(bc["lcmQP"])
                        elif idx == 5:
                            txt = str(bc["r"][0])
                        elif idx == 6:
                            txt = str(round(bc["sigma_d"], 3))

                        self.table.setItem(iQ, iP, QTableWidgetItem(txt))
                        q = bc["q"]

                        if self.data[iQ][iP].get_windingstep() == 1:
                            self.table.item(iQ, iP).setBackground(
                                QtGui.QColor("#ADD8E6")
                            )
                        elif q.denominator == 1:
                            self.table.item(iQ, iP).setBackground(
                                QtGui.QColor("#E6C3AD")
                            )
                        else:
                            self.table.item(iQ, iP).setBackground(
                                QtGui.QColor("#BCFFBC")
                            )
                        if self.data[iQ][iP].get_num_empty_slots() != 0:
                            self.table.item(iQ, iP).setBackground(
                                QtGui.QColor("#E9AEE2")
                            )
                        self.table.item(iQ, iP).setFlags(
                            QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled
                        )  # note editable

        for iP, kP in enumerate(self.Plist):
            self.table.resizeColumnToContents(iP)
            #  self.table.setColumnWidth(k1, 25)

        # reset table with winding layout because there is noting choosed
        self.tableWindingLayout.clear()
        self.tableWindingLayout.setRowCount(0)
        self.tableWindingLayout.setColumnCount(0)

    def combination_selected(self):
        """
        user selected a cell/winding combination
        """
        sel = self.tableCombinations.selectedIndexes()
        if sel:
            sel = sel[0]
            row = sel.row()
            column = sel.column()
            d = self.data[row][column]
            bc, bc_text = d.get_basic_characteristics()
            self.textBrowser_wdginfo.setHtml(bc_text)

            self.table = self.tableWindingLayout
            self.table.clear()
            self.table.setRowCount(self.layers)
            self.table.setColumnCount(d.get_num_slots())

            l, ls, lcol = d.get_layers()
            for k1 in range(np.shape(l)[0]):
                for k2 in range(np.shape(l)[1]):
                    self.table.setItem(k1, k2, QTableWidgetItem(ls[k1, k2]))
                    item = self.table.item(k1, k2)
                    if item:
                        item.setBackground(QtGui.QColor(lcol[k1, k2]))
                    self.table.item(k1, k2).setFlags(
                        QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled
                    )

            for k1 in range(d.get_num_slots()):
                self.table.resizeColumnToContents(k1)
            return row, column
        else:
            return None, None

    def run(self):
        self.generate()
        ok = self.exec_()
        if ok:
            if self.radioButton_overwrite_winding.isChecked():
                overwrite = True
            else:
                overwrite = False
            row, column = self.combination_selected()
            if row is not None:
                ret = {}
                ret["Q"] = self.data[row][column].get_num_slots()
                ret["P"] = 2 * self.data[row][column].get_num_polepairs()
                ret["m"] = self.data[row][column].get_num_phases()
                ret["w"] = self.data[row][column].get_windingstep()
                ret["layers"] = self.layers
                ret["Qes"] = self.data[row][column].get_num_empty_slots()
                ret["overwrite"] = overwrite
                return ret
            else:
                return None
        else:
            return None



# -*- coding: utf-8 -*-

import os
import sys

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))

from swat_em.datamodel import datamodel
import numpy as np


def test_get_layers():
    data = datamodel()
    data.genwdg(Q=12, P=10, m=3, w=1, layers=2)
    data.analyse_wdg()
    #  bc, txt = data.get_basic_characteristics()
    #  print(bc)

    l, ls, lcol = data.get_layers()
    #  print(l)
    #  print(ls)
    #  print(lcol)
    assert list(l[0, :]) == [1, 2, -2, -3, 3, 1, -1, -2, 2, 3, -3, -1]  # first layer
    assert list(l[1, :]) == [1, -1, -2, 2, 3, -3, -1, 1, 2, -2, -3, 3]  # second layer


if __name__ == "__main__":
    test_get_layers()
